@@ -1,5 +1,4 @@
-#!/usr/bin/perl
-# $MirOS: src/bin/mksh/check.pl,v 1.8 2006/05/08 11:59:41 tg Exp $
+# $MirOS: src/bin/mksh/check.pl,v 1.10 2007/02/13 12:59:03 tg Exp $
 # $OpenBSD: th,v 1.12 2005/05/28 04:53:47 millert Exp $
 #-
 # Example test:
@@ -843,14 +842,14 @@ read_test
 	last if /^\s*---\s*$/;
 	$start_lineno = $. if !defined $start_lineno;
 	if (!/^([-\w]+):\s*(|\S|\S.*\S)\s*$/) {
-	    print STDERR "$prog:$file:$.: unrecognized line\n";
+	    print STDERR "$prog:$file:$.: unrecognised line\n";
 	    return undef;
 	}
 	($field, $val) = ($1, $2);
 	$sfield = $field;
 	$flags = $test_fields{$field};
 	if (!defined $flags) {
-	    print STDERR "$prog:$file:$.: unrecognized field \"$field\"\n";
+	    print STDERR "$prog:$file:$.: unrecognised field \"$field\"\n";
 	    return undef;
 	}
 	if ($flags =~ /s/) {
