@@ -2,7 +2,11 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/eval.c,v 1.13 2006/08/01 13:43:26 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/eval.c,v 1.25.2.1 2007/05/13 19:29:34 tg Exp $");
+
+#ifdef MKSH_SMALL
+#define MKSH_NOPWNAM
+#endif
 
 /*
  * string expansion
@@ -13,14 +17,14 @@ __RCSID("$MirOS: src/bin/mksh/eval.c,v 1.13 2006/08/01 13:43:26 tg Exp $");
 
 /* expansion generator state */
 typedef struct Expand {
-	/* int  type; */	/* see expand() */
+	/* int type; */		/* see expand() */
 	const char *str;	/* string */
 	union {
 		const char **strv;/* string[] */
-		struct shf *shf;/* file */
+		struct shf *shf;  /* file */
 	} u;			/* source */
 	struct tbl *var;	/* variable in ${var..} */
-	short	split;		/* split "$@" / call waitlast $() */
+	short split;		/* split "$@" / call waitlast $() */
 } Expand;
 
 #define	XBASE		0	/* scanning original */
@@ -36,14 +40,16 @@ typedef struct Expand {
 #define IFS_WS		1	/* have seen IFS white-space */
 #define IFS_NWS		2	/* have seen IFS non-white-space */
 
-static int varsub(Expand *, char *, char *, int *, int *);
-static int comsub(Expand *, char *);
+static int varsub(Expand *, const char *, const char *, int *, int *);
+static int comsub(Expand *, const char *);
 static char *trimsub(char *, char *, int);
 static void glob(char *, XPtrV *, int);
 static void globit(XString *, char **, char *, XPtrV *, int);
-static char *maybe_expand_tilde(char *, XString *, char **, int);
+static const char *maybe_expand_tilde(const char *, XString *, char **, int);
 static char *tilde(char *);
+#ifndef MKSH_NOPWNAM
 static char *homedir(char *);
+#endif
 static void alt_expand(XPtrV *, char *, char *, char *, int);
 
 /* compile and expand word */
@@ -57,7 +63,7 @@ substitute(const char *cp, int f)
 	s->start = s->str = cp;
 	source = s;
 	if (yylex(ONEWORD) != LWORD)
-		internal_errorf(1, "substitute");
+		internal_errorf("substitute");
 	source = sold;
 	afree(s, ATEMP);
 	return evalstr(yylval.cp, f);
@@ -67,12 +73,16 @@ substitute(const char *cp, int f)
  * expand arg-list
  */
 char **
-eval(char **ap, int f)
+eval(const char **ap, int f)
 {
 	XPtrV w;
 
-	if (*ap == NULL)
-		return ap;
+	if (*ap == NULL) {
+		union mksh_ccphack vap;
+
+		vap.ro = ap;
+		return (vap.rw);
+	}
 	XPinit(w, 32);
 	XPput(w, NULL);		/* space for shell name */
 	while (*ap != NULL)
@@ -85,15 +95,16 @@ eval(char **ap, int f)
  * expand string
  */
 char *
-evalstr(char *cp, int f)
+evalstr(const char *cp, int f)
 {
 	XPtrV w;
+	char *dp;
 
 	XPinit(w, 1);
 	expand(cp, &w, f);
-	cp = (XPsize(w) == 0) ? null : (char*) *XPptrv(w);
+	dp = (XPsize(w) == 0) ? null : (char*) *XPptrv(w);
 	XPfree(w);
-	return cp;
+	return (dp);
 }
 
 /*
@@ -101,25 +112,26 @@ evalstr(char *cp, int f)
  * used from iosetup to expand redirection files
  */
 char *
-evalonestr(char *cp, int f)
+evalonestr(const char *cp, int f)
 {
 	XPtrV w;
+	char *rv;
 
 	XPinit(w, 1);
 	expand(cp, &w, f);
 	switch (XPsize(w)) {
 	case 0:
-		cp = null;
+		rv = null;
 		break;
 	case 1:
-		cp = (char*) *XPptrv(w);
+		rv = (char *) *XPptrv(w);
 		break;
 	default:
-		cp = evalstr(cp, f&~DOGLOB);
+		rv = evalstr(cp, f&~DOGLOB);
 		break;
 	}
 	XPfree(w);
-	return cp;
+	return (rv);
 }
 
 /* for nested substitution: ${var:=$var2} */
@@ -134,7 +146,7 @@ typedef struct SubType {
 } SubType;
 
 void
-expand(char *cp,	/* input word */
+expand(const char *cp,	/* input word */
     XPtrV *wp,		/* output words */
     int f)		/* DO* flags */
 {
@@ -142,7 +154,8 @@ expand(char *cp,	/* input word */
 	int type;		/* expansion type */
 	int quote = 0;		/* quoted */
 	XString ds;		/* destination string */
-	char *dp, *sp;		/* dest., source */
+	char *dp;		/* destination */
+	const char *sp;		/* source */
 	int fdo, word;		/* second pass flags; have word */
 	int doblank;		/* field splitting of parameter/command subst */
 	Expand x = {		/* expansion variables */
@@ -155,7 +168,7 @@ expand(char *cp,	/* input word */
 	size_t len;
 
 	if (cp == NULL)
-		internal_errorf(1, "expand(NULL)");
+		internal_errorf("expand(NULL)");
 	/* for alias, readonly, set, typeset commands */
 	if ((f & DOVACHECK) && is_wdvarassign(cp)) {
 		f &= ~(DOVACHECK|DOBLANK|DOGLOB|DOTILDE);
@@ -219,7 +232,7 @@ expand(char *cp,	/* input word */
 					type = comsub(&x, sp);
 					if (type == XCOM && (f&DOBLANK))
 						doblank++;
-					sp = strchr(sp, 0) + 1;
+					sp = strnul(sp) + 1;
 					newlines = 0;
 				}
 				continue;
@@ -242,7 +255,7 @@ expand(char *cp,	/* input word */
 					v.name[0] = '\0';
 					v_evaluate(&v, substitute(sp, 0),
 					    KSH_UNWIND_ERROR, true);
-					sp = strchr(sp, 0) + 1;
+					sp = strnul(sp) + 1;
 					for (p = str_val(&v); *p; ) {
 						Xcheck(ds, dp);
 						*dp++ = *p++;
@@ -256,26 +269,26 @@ expand(char *cp,	/* input word */
 			   * This is where all syntax checking gets done...
 			   */
 			    {
-				char *varname = ++sp; /* skip the { or x (}) */
+				const char *varname = ++sp; /* skip the { or x (}) */
 				int stype;
 				int slen = 0;
 
-				sp = strchr(sp, '\0') + 1; /* skip variable */
+				sp = cstrchr(sp, '\0') + 1; /* skip variable */
 				type = varsub(&x, varname, sp, &stype, &slen);
 				if (type < 0) {
-					char endc;
-					char *str, *end;
+					char *beg, *end, *str;
 
 					sp = varname - 2; /* restore sp */
-					end = (char *) wdscan(sp, CSUBST);
+					end = (beg = wdcopy(sp, ATEMP)) +
+					    (wdscan(sp, CSUBST) - sp);
 					/* ({) the } or x is already skipped */
-					endc = *end;
-					*end = EOS;
-					str = snptreef(NULL, 64, "%S", sp);
-					*end = endc;
+					if (end < wdscan(beg, EOS))
+						*end = EOS;
+					str = snptreef(NULL, 64, "%S", beg);
+					afree(beg, ATEMP);
 					errorf("%s: bad substitution", str);
 				}
-				if (f&DOBLANK)
+				if (f & DOBLANK)
 					doblank++;
 				tilde_ok = 0;
 				if (type == XBASE) {	/* expand? */
@@ -345,7 +358,7 @@ expand(char *cp,	/* input word */
 					}
 				} else
 					/* skip word */
-					sp = (char *) wdscan(sp, CSUBST);
+					sp += wdscan(sp, CSUBST) - sp;
 				continue;
 			    }
 			case CSUBST: /* only get here if expanding word */
@@ -565,8 +578,9 @@ expand(char *cp,	/* input word */
 				    Xlength(ds, dp) == 0) {
 					char *p;
 
-					if ((p = strdup("")) == NULL)
-						internal_errorf(1, "unable "
+					if ((p = str_nsave(null, 0, ATEMP))
+					    == NULL)
+						internal_errorf("unable "
 						    "to allocate memory");
 					XPput(*wp, p);
 				}
@@ -632,7 +646,8 @@ expand(char *cp,	/* input word */
 					if (type == XBASE &&
 					    (f & (DOTILDE|DOASNTILDE)) &&
 					    (tilde_ok & 2)) {
-						char *p, *dp_x;
+						const char *p;
+						char *dp_x;
 
 						dp_x = dp;
 						p = maybe_expand_tilde(sp,
@@ -669,7 +684,7 @@ expand(char *cp,	/* input word */
  * Prepare to generate the string returned by ${} substitution.
  */
 static int
-varsub(Expand *xp, char *sp, char *word,
+varsub(Expand *xp, const char *sp, const char *word,
     int *stypep,	/* becomes qualifier type */
     int *slenp)		/* " " len (=, :=, etc.) valid iff *stypep != 0 */
 {
@@ -677,7 +692,7 @@ varsub(Expand *xp, char *sp, char *word,
 	int state;	/* next state: XBASE, XARG, XSUB, XNULLSUB */
 	int stype;	/* substitution type */
 	int slen;
-	char *p;
+	const char *p;
 	struct tbl *vp;
 
 	if (sp[0] == '\0')	/* Bad variable name */
@@ -694,7 +709,8 @@ varsub(Expand *xp, char *sp, char *word,
 			return -1;
 		sp++;
 		/* Check for size of array */
-		if ((p=strchr(sp,'[')) && (p[1]=='*'||p[1]=='@') && p[2]==']') {
+		if ((p = cstrchr(sp, '[')) && (p[1] == '*' || p[1] == '@') &&
+		    p[2] == ']') {
 			int n = 0;
 			int max = 0;
 
@@ -717,7 +733,7 @@ varsub(Expand *xp, char *sp, char *word,
 		if (Flag(FNOUNSET) && c == 0 && !zero_ok)
 			errorf("%s: parameter not set", sp);
 		*stypep = 0; /* unqualified variable/string substitution */
-		xp->str = str_save(ulton((unsigned long)c, 10), ATEMP);
+		xp->str = shf_smprintf("%lu", (unsigned long)c);
 		return XSUB;
 	}
 
@@ -765,7 +781,8 @@ varsub(Expand *xp, char *sp, char *word,
 			state = XARG;
 		}
 	} else {
-		if ((p=strchr(sp,'[')) && (p[1]=='*'||p[1]=='@') && p[2]==']') {
+		if ((p = cstrchr(sp, '[')) && (p[1] == '*' || p[1] == '@') &&
+		    p[2] == ']') {
 			XPtrV wv;
 
 			switch (stype & 0x7f) {
@@ -796,7 +813,7 @@ varsub(Expand *xp, char *sp, char *word,
 		} else {
 			/* Can't assign things like $! or $1 */
 			if ((stype & 0x7f) == '=' &&
-			    (ctype(*sp, C_VAR1) || digit(*sp)))
+			    ctype(*sp, C_VAR1 | C_DIGIT))
 				return -1;
 			xp->var = global(sp);
 			xp->str = str_val(xp->var);
@@ -820,7 +837,7 @@ varsub(Expand *xp, char *sp, char *word,
  * Run the command in $(...) and read its output.
  */
 static int
-comsub(Expand *xp, char *cp)
+comsub(Expand *xp, const char *cp)
 {
 	Source *s, *sold;
 	struct op *t;
@@ -874,7 +891,7 @@ comsub(Expand *xp, char *cp)
 static char *
 trimsub(char *str, char *pat, int how)
 {
-	char *end = strchr(str, 0);
+	char *end = strnul(str);
 	char *p, c;
 
 	switch (how&0xff) {	/* UCHAR_MAX maybe? */
@@ -929,8 +946,8 @@ glob(char *cp, XPtrV *wp, int markdirs)
 	if (glob_str(cp, wp, markdirs) == 0)
 		XPput(*wp, debunk(cp, cp, strlen(cp) + 1));
 	else
-		qsortp(XPptrv(*wp) + oldsize, (size_t)(XPsize(*wp) - oldsize),
-			xstrcmp);
+		qsort(XPptrv(*wp) + oldsize, XPsize(*wp) - oldsize,
+		    sizeof (void *), xstrcmp);
 }
 
 #define GF_NONE		0
@@ -1086,15 +1103,16 @@ globit(XString *xs,	/* dest string */
 char *
 debunk(char *dp, const char *sp, size_t dlen)
 {
-	char *d, *s;
+	char *d;
+	const char *s;
 
-	if ((s = strchr(sp, MAGIC))) {
+	if ((s = cstrchr(sp, MAGIC))) {
 		if (s - sp >= (ssize_t)dlen)
 			return dp;
 		memcpy(dp, sp, s - sp);
 		for (d = dp + (s - sp); *s && (d - dp < (ssize_t)dlen); s++)
 			if (!ISMAGIC(*s) || !(*++s & 0x80) ||
-			    !strchr("*+?@! ", *s & 0x7f))
+			    !vstrchr("*+?@! ", *s & 0x7f))
 				*d++ = *s;
 			else {
 				/* extended pattern operators: *+?@! */
@@ -1113,12 +1131,13 @@ debunk(char *dp, const char *sp, size_t dlen)
  * puts the expanded version in *dcp,dp and returns a pointer in p just
  * past the name, otherwise returns 0.
  */
-static char *
-maybe_expand_tilde(char *p, XString *dsp, char **dpp, int isassign)
+static const char *
+maybe_expand_tilde(const char *p, XString *dsp, char **dpp, int isassign)
 {
 	XString ts;
 	char *dp = *dpp;
-	char *tp, *r;
+	char *tp;
+	const char *r;
 
 	Xinit(ts, tp, 16, ATEMP);
 	/* : only for DOASNTILDE form */
@@ -1154,7 +1173,7 @@ maybe_expand_tilde(char *p, XString *dsp, char **dpp, int isassign)
 static char *
 tilde(char *cp)
 {
-	char *dp;
+	char *dp = null;
 
 	if (cp[0] == '\0')
 		dp = str_val(global("HOME"));
@@ -1162,21 +1181,23 @@ tilde(char *cp)
 		dp = str_val(global("PWD"));
 	else if (cp[0] == '-' && cp[1] == '\0')
 		dp = str_val(global("OLDPWD"));
+#ifndef MKSH_NOPWNAM
 	else
 		dp = homedir(cp);
+#endif
 	/* If HOME, PWD or OLDPWD are not set, don't expand ~ */
 	if (dp == null)
 		dp = NULL;
 	return dp;
 }
 
+#ifndef MKSH_NOPWNAM
 /*
  * map userid to user's home directory.
  * note that 4.3's getpw adds more than 6K to the shell,
  * and the YP version probably adds much more.
  * we might consider our own version of getpwnam() to keep the size down.
  */
-
 static char *
 homedir(char *name)
 {
@@ -1194,6 +1215,7 @@ homedir(char *name)
 	}
 	return ap->val.s;
 }
+#endif
 
 static void
 alt_expand(XPtrV *wp, char *start, char *exp_start, char *end, int fdo)
