@@ -2,21 +2,19 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/exec.c,v 1.16 2006/08/01 13:43:26 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/exec.c,v 1.29.2.1 2007/05/13 19:29:34 tg Exp $");
 
-static int	comexec(struct op *, struct tbl *volatile, char **,
-		    int volatile);
-static void scriptexec(struct op *, char **) __attribute__((noreturn));
-static int	call_builtin(struct tbl *, char **);
-static int	iosetup(struct ioword *, struct tbl *);
-static int	herein(const char *, int);
-static char	*do_selectargs(char **, bool);
-static int	dbteste_isa(Test_env *, Test_meta);
+static int comexec(struct op *, struct tbl *volatile, const char **,
+    int volatile);
+static void scriptexec(struct op *, const char **)
+    __attribute__((noreturn));
+static int call_builtin(struct tbl *, const char **);
+static int iosetup(struct ioword *, struct tbl *);
+static int herein(const char *, int);
+static const char *do_selectargs(const char **, bool);
+static int dbteste_isa(Test_env *, Test_meta);
 static const char *dbteste_getopnd(Test_env *, Test_op, int);
-static int	dbteste_eval(Test_env *, Test_op, const char *, const char *,
-		    int);
-static void	dbteste_error(Test_env *, int, const char *);
-
+static void dbteste_error(Test_env *, int, const char *);
 
 /*
  * execute command tree
@@ -28,8 +26,9 @@ execute(struct op *volatile t,
 	int i;
 	volatile int rv = 0;
 	int pv[2];
-	char ** volatile ap;
-	char *s, *cp;
+	const char ** volatile ap;
+	char ** volatile up;
+	const char *s, *cp;
 	struct ioword **iowp;
 	struct tbl *tp = NULL;
 
@@ -62,10 +61,11 @@ execute(struct op *volatile t,
 		/* POSIX says expand command words first, then redirections,
 		 * and assignments last..
 		 */
-		ap = eval(t->args, t->u.evalflags | DOBLANK | DOGLOB | DOTILDE);
+		up = eval(t->args, t->u.evalflags | DOBLANK | DOGLOB | DOTILDE);
 		if (flags & XTIME)
 			/* Allow option parsing (bizarre, but POSIX) */
-			timex_hook(t, &ap);
+			timex_hook(t, &up);
+		ap = (const char **)up;
 		if (Flag(FXTRACE) && ap[0]) {
 			shf_fprintf(shl_out, "%s",
 				substitute(str_val(global("PS4")), 0));
@@ -81,7 +81,7 @@ execute(struct op *volatile t,
 
 	if (t->ioact != NULL || t->type == TPIPE || t->type == TCOPROC) {
 		e->savefd = (short *) alloc(sizeofN(short, NUFILE), ATEMP);
-		/* initialize to not redirected */
+		/* initialise to not redirected */
 		memset(e->savefd, 0, sizeofN(short, NUFILE));
 	}
 
@@ -103,7 +103,7 @@ execute(struct op *volatile t,
 
 	switch (t->type) {
 	case TCOM:
-		rv = comexec(t, tp, ap, flags);
+		rv = comexec(t, tp, (const char **)ap, flags);
 		break;
 
 	case TPAREN:
@@ -206,7 +206,7 @@ execute(struct op *volatile t,
 
 	case TASYNC:
 		/* XXX non-optimal, I think - "(foo &)", forks for (),
-		 * forks again for async...  parent should optimize
+		 * forks again for async...  parent should optimise
 		 * this to "foo &"...
 		 */
 		rv = execute(t->left, (flags&~XEXEC)|XBGND|XFORK);
@@ -233,7 +233,7 @@ execute(struct op *volatile t,
 		te.pos.wp = t->args;
 		te.isa = dbteste_isa;
 		te.getopnd = dbteste_getopnd;
-		te.eval = dbteste_eval;
+		te.eval = test_eval;
 		te.error = dbteste_error;
 
 		rv = test_parse(&te);
@@ -244,8 +244,9 @@ execute(struct op *volatile t,
 	case TSELECT:
 	    {
 		volatile bool is_first = true;
-		ap = (t->vars != NULL) ? eval(t->vars, DOBLANK|DOGLOB|DOTILDE) :
-		    e->loc->argv + 1;
+		ap = (t->vars == NULL) ? e->loc->argv + 1 :
+		    (const char **)eval((const char **)t->vars,
+		    DOBLANK | DOGLOB | DOTILDE);
 		e->type = E_LOOP;
 		while (1) {
 			i = sigsetjmp(e->jbuf, 0);
@@ -313,7 +314,7 @@ execute(struct op *volatile t,
 	case TCASE:
 		cp = evalstr(t->str, DOTILDE);
 		for (t = t->left; t != NULL && t->type == TPAT; t = t->right)
-		    for (ap = t->vars; *ap; ap++)
+		    for (ap = (const char **)t->vars; *ap; ap++)
 			if ((s = evalstr(*ap, DOTILDE|DOPAT)) &&
 			    gmatchx(cp, s, false))
 				goto Found;
@@ -339,12 +340,17 @@ execute(struct op *volatile t,
 
 	case TEXEC:		/* an eval'd TCOM */
 		s = t->args[0];
-		ap = makenv();
+		up = makenv();
 		restoresigs();
 		cleanup_proc_env();
-		execve(t->str, t->args, ap);
+		{
+			union mksh_ccphack cargs;
+
+			cargs.ro = t->args;
+			execve(t->str, cargs.rw, up);
+		}
 		if (errno == ENOEXEC)
-			scriptexec(t, ap);
+			scriptexec(t, (const char **)up);
 		else
 			errorf("%s: %s", s, strerror(errno));
 	}
@@ -367,12 +373,13 @@ execute(struct op *volatile t,
  */
 
 static int
-comexec(struct op *t, struct tbl *volatile tp, char **ap, volatile int flags)
+comexec(struct op *t, struct tbl *volatile tp, const char **ap,
+    volatile int flags)
 {
 	int i;
 	volatile int rv = 0;
-	char *cp;
-	char **lastp;
+	const char *cp;
+	const char **lastp;
 	static struct op texec; /* Must be static (XXX but why?) */
 	int type_flags;
 	int keepasn_ok;
@@ -492,7 +499,7 @@ comexec(struct op *t, struct tbl *volatile tp, char **ap, volatile int flags)
 		rv = subst_exstat;
 		goto Leave;
 	} else if (!tp) {
-		if (Flag(FRESTRICTED) && strchr(cp, '/')) {
+		if (Flag(FRESTRICTED) && vstrchr(cp, '/')) {
 			warningf(true, "%s: restricted", cp);
 			rv = 1;
 			goto Leave;
@@ -502,14 +509,14 @@ comexec(struct op *t, struct tbl *volatile tp, char **ap, volatile int flags)
 
 	switch (tp->type) {
 	case CSHELL:			/* shell built-in */
-		rv = call_builtin(tp, ap);
+		rv = call_builtin(tp, (const char **)ap);
 		break;
 
 	case CFUNC:			/* function call */
 	    {
 		volatile int old_xflag;
 		volatile Tflag old_inuse;
-		char *volatile old_kshname;
+		const char *volatile old_kshname;
 
 		if (!(tp->flag & ISSET)) {
 			struct tbl *ftp;
@@ -554,7 +561,7 @@ comexec(struct op *t, struct tbl *volatile tp, char **ap, volatile int flags)
 		if (tp->flag & FKSH)
 			kshname = ap[0];
 		else
-			ap[0] = (char *) kshname;
+			ap[0] = kshname;
 		e->loc->argv = ap;
 		for (i = 0; *ap++ != NULL; i++)
 			;
@@ -609,7 +616,7 @@ comexec(struct op *t, struct tbl *volatile tp, char **ap, volatile int flags)
 			/* NOTREACHED */
 		default:
 			quitenv(NULL);
-			internal_errorf(1, "CFUNC %d", i);
+			internal_errorf("CFUNC %d", i);
 		}
 		break;
 	    }
@@ -663,34 +670,93 @@ comexec(struct op *t, struct tbl *volatile tp, char **ap, volatile int flags)
 }
 
 static void
-scriptexec(struct op *tp, char **ap)
+scriptexec(struct op *tp, const char **ap)
 {
-	char *sh;
+	const char *sh;
+#ifndef MKSH_SMALL
+	unsigned char *cp;
+	char buf[64];		/* 64 == MAXINTERP in MirBSD <sys/param.h> */
+	int fd;
+#endif
+	union mksh_ccphack args, cap;
 
-	sh = str_val(global(EXECSHELL_STR));
+	sh = str_val(global("EXECSHELL"));
 	if (sh && *sh)
 		sh = search(sh, path, X_OK, NULL);
 	if (!sh || !*sh)
-		sh = strdup(EXECSHELL);
+		sh = "/bin/sh";
 
 	*tp->args-- = tp->str;
-	*tp->args = sh;
 
-	execve(tp->args[0], tp->args, ap);
+#ifndef MKSH_SMALL
+	if ((fd = open(tp->str, O_RDONLY)) >= 0) {
+		/* read first MAXINTERP octets from file */
+		if (read(fd, buf, sizeof (buf)) <= 0)
+			/* read error -> no good */
+			buf[0] = '\0';
+		close(fd);
+		/* scan for newline (or CR) or NUL _before_ end of buffer */
+		cp = (unsigned char *)buf;
+		while ((char *)cp < (buf + sizeof (buf)))
+			if (*cp == '\0' || *cp == '\n' || *cp == '\r') {
+				*cp = '\0';
+				break;
+			} else
+				++cp;
+		/* if the shebang line is longer than MAXINTERP, bail out */
+		if ((char *)cp >= (buf + sizeof (buf)))
+			goto noshebang;
+		/* skip UTF-8 Byte Order Mark, if present */
+		cp = (unsigned char *)buf;
+		if ((cp[0] == 0xEF) && (cp[1] == 0xBB) && (cp[2] == 0xBF))
+			cp += 3;
+		/* bail out if read error (above) or no shebang */
+		if ((cp[0] != '#') || (cp[1] != '!'))
+			goto noshebang;
+		cp += 2;
+		/* skip whitespace before shell name */
+		while (*cp == ' ' || *cp == '\t')
+			++cp;
+		/* just whitespace on the line? */
+		if (*cp == '\0')
+			goto noshebang;
+		/* no, we actually found an interpreter name */
+		sh = (char *)cp;
+		/* look for end of shell/interpreter name */
+		while (*cp != ' ' && *cp != '\t' && *cp != '\0')
+			++cp;
+		/* any arguments? */
+		if (*cp) {
+			*cp++ = '\0';
+			/* skip spaces before arguments */
+			while (*cp == ' ' || *cp == '\t')
+				++cp;
+			/* pass it all in ONE argument (historic reasons) */
+			if (*cp)
+				*tp->args-- = (char *)cp;
+		}
+	}
+ noshebang:
+#endif
+	args.ro = tp->args;
+	*args.ro = sh;
 
-	/* report both the program that was run and the bogus shell */
+	cap.ro = ap;
+	execve(args.rw[0], args.rw, cap.rw);
+
+	/* report both the programme that was run and the bogus interpreter */
 	errorf("%s: %s: %s", tp->str, sh, strerror(errno));
 }
 
 int
-shcomexec(char **wp)
+shcomexec(const char **wp)
 {
 	struct tbl *tp;
 
 	tp = ktsearch(&builtins, *wp, hash(*wp));
 	if (tp == NULL)
-		internal_errorf(1, "shcomexec: %s", *wp);
-	return call_builtin(tp, wp);
+		internal_errorf("shcomexec: %s", *wp);
+	return (call_builtin(tp, wp));
 }
 
 /*
@@ -766,7 +832,7 @@ define(const char *name, struct op *t)
  * add builtin
  */
 void
-builtin(const char *name, int (*func) (char **))
+builtin(const char *name, int (*func) (const char **))
 {
 	struct tbl *tp;
 	Tflag flag;
@@ -801,9 +867,9 @@ findcom(const char *name, int flags)
 	struct tbl *tp = NULL, *tbi;
 	int insert = Flag(FTRACKALL);	/* insert if not found */
 	char *fpath;			/* for function autoloading */
-	char *npath;
+	const char *npath;
 
-	if (strchr(name, '/') != NULL) {
+	if (vstrchr(name, '/')) {
 		insert = 0;
 		/* prevent FPATH search below */
 		flags &= ~FC_FUNC;
@@ -832,7 +898,7 @@ findcom(const char *name, int flags)
 		tp = tbi;
 	if (!tp && (flags & FC_PATH) && !(flags & FC_DEFPATH)) {
 		tp = ktsearch(&taliases, name, h);
-		if (tp && (tp->flag & ISSET) && eaccess(tp->val.s, X_OK) != 0) {
+		if (tp && (tp->flag & ISSET) && access(tp->val.s, X_OK) != 0) {
 			if (tp->flag & ALLOC) {
 				tp->flag &= ~ALLOC;
 				afree(tp->val.s, APERM);
@@ -857,7 +923,7 @@ findcom(const char *name, int flags)
 		npath = search(name, flags & FC_DEFPATH ? def_path : path,
 		    X_OK, &tp->u2.errno_);
 		if (npath) {
-			tp->val.s = tp == &temp ? npath : str_save(npath, APERM);
+			tp->val.s = str_save(npath, APERM);
 			tp->flag |= ISSET|ALLOC;
 		} else if ((flags & FC_FUNC) &&
 		    (fpath = str_val(global("FPATH"))) != null &&
@@ -906,7 +972,7 @@ search_access(const char *lpath, int mode,
 
 	if (stat(lpath, &statb) < 0)
 		return -1;
-	ret = eaccess(lpath, mode);
+	ret = access(lpath, mode);
 	if (ret < 0)
 		err = errno; /* File exists, but we can't access it */
 	else if (mode == X_OK && (!S_ISREG(statb.st_mode) ||
@@ -923,7 +989,7 @@ search_access(const char *lpath, int mode,
 /*
  * search for command with PATH
  */
-char *
+const char *
 search(const char *name, const char *lpath,
     int mode,		/* R_OK or X_OK */
     int *errnop)	/* set if candidate found, but not suitable */
@@ -935,9 +1001,9 @@ search(const char *name, const char *lpath,
 
 	if (errnop)
 		*errnop = 0;
-	if (strchr(name, '/')) {
+	if (vstrchr(name, '/')) {
 		if (search_access(name, mode, errnop) == 0)
-			return (char *) name;
+			return (name);
 		return NULL;
 	}
 
@@ -947,7 +1013,7 @@ search(const char *name, const char *lpath,
 	sp = lpath;
 	while (sp != NULL) {
 		xp = Xstring(xs, xp);
-		if (!(p = strchr(sp, ':')))
+		if (!(p = cstrchr(sp, ':')))
 			p = sp + strlen(sp);
 		if (p != sp) {
 			XcheckN(xs, xp, p - sp);
@@ -968,7 +1034,7 @@ search(const char *name, const char *lpath,
 }
 
 static int
-call_builtin(struct tbl *tp, char **wp)
+call_builtin(struct tbl *tp, const char **wp)
 {
 	int rv;
 
@@ -1169,7 +1235,7 @@ herein(const char *content, int sub)
 		s->start = s->str = content;
 		source = s;
 		if (yylex(ONEWORD|HEREDOC) != LWORD)
-			internal_errorf(1, "herein: yylex");
+			internal_errorf("herein: yylex");
 		source = osource;
 		shf_puts(evalstr(yylval.cp, 0), shf);
 	} else
@@ -1191,10 +1257,10 @@ herein(const char *content, int sub)
  *	ksh special - the select command processing section
  *	print the args in column form - assuming that we can
  */
-static char *
-do_selectargs(char **ap, bool print_menu)
+static const char *
+do_selectargs(const char **ap, bool print_menu)
 {
-	static const char *const read_args[] = {
+	static const char *read_args[] = {
 		"read", "-r", "REPLY", NULL
 	};
 	char *s;
@@ -1211,11 +1277,11 @@ do_selectargs(char **ap, bool print_menu)
 		if (print_menu || !*str_val(global("REPLY")))
 			pr_menu(ap);
 		shellf("%s", str_val(global("PS3")));
-		if (call_builtin(findcom("read", FC_BI), (char **) read_args))
+		if (call_builtin(findcom("read", FC_BI), read_args))
 			return NULL;
 		s = str_val(global("REPLY"));
 		if (*s) {
-			i = atoi(s);
+			getn(s, &i);
 			return (i >= 1 && i <= argct) ? ap[i - 1] : null;
 		}
 		print_menu = 1;
@@ -1223,18 +1289,19 @@ do_selectargs(char **ap, bool print_menu)
 }
 
 struct select_menu_info {
-	char	*const *args;
-	int	arg_width;
-	int	num_width;
+	const char *const *args;
+	int arg_width;
+	int num_width;
 };
 
-static char *select_fmt_entry(void *arg, int i, char *buf, int buflen);
+static char *select_fmt_entry(const void *, int, char *, int);
 
 /* format a single select menu item */
 static char *
-select_fmt_entry(void *arg, int i, char *buf, int buflen)
+select_fmt_entry(const void *arg, int i, char *buf, int buflen)
 {
-	struct select_menu_info *smi = (struct select_menu_info *) arg;
+	const struct select_menu_info *smi =
+	    (const struct select_menu_info *)arg;
 
 	shf_snprintf(buf, buflen, "%*d) %s",
 	    smi->num_width, i + 1, smi->args[i]);
@@ -1245,10 +1312,10 @@ select_fmt_entry(void *arg, int i, char *buf, int buflen)
  *	print a select style menu
  */
 int
-pr_menu(char *const *ap)
+pr_menu(const char *const *ap)
 {
 	struct select_menu_info smi;
-	char *const *pp;
+	const char *const *pp;
 	int nwidth, dwidth;
 	int i, n;
 
@@ -1285,10 +1352,10 @@ pr_menu(char *const *ap)
 
 /* XXX: horrible kludge to fit within the framework */
 
-static char *plain_fmt_entry(void *arg, int i, char *buf, int buflen);
+static char *plain_fmt_entry(const void *, int, char *, int);
 
 static char *
-plain_fmt_entry(void *arg, int i, char *buf, int buflen)
+plain_fmt_entry(const void *arg, int i, char *buf, int buflen)
 {
 	shf_snprintf(buf, buflen, "%s", ((char *const *)arg)[i]);
 	return buf;
@@ -1305,7 +1372,7 @@ pr_list(char *const *ap)
 		i = strlen(*pp);
 		nwidth = (i > nwidth) ? i : nwidth;
 	}
-	print_columns(shl_out, n, plain_fmt_entry, (void *) ap, nwidth + 1, 0);
+	print_columns(shl_out, n, plain_fmt_entry, (const void *)ap, nwidth + 1, 0);
 
 	return n;
 }
@@ -1326,7 +1393,7 @@ dbteste_isa(Test_env *te, Test_meta meta)
 {
 	int ret = 0;
 	int uqword;
-	char *p;
+	const char *p;
 
 	if (!*te->pos.wp)
 		return meta == TM_END;
@@ -1362,7 +1429,7 @@ dbteste_isa(Test_env *te, Test_meta meta)
 static const char *
 dbteste_getopnd(Test_env *te, Test_op op, int do_eval)
 {
-	char *s = *te->pos.wp;
+	const char *s = *te->pos.wp;
 
 	if (!s)
 		return NULL;
@@ -1380,16 +1447,9 @@ dbteste_getopnd(Test_env *te, Test_op op, int do_eval)
 	return s;
 }
 
-static int
-dbteste_eval(Test_env *te, Test_op op, const char *opnd1, const char *opnd2,
-    int do_eval)
-{
-	return test_eval(te, op, opnd1, opnd2, do_eval);
-}
-
 static void
 dbteste_error(Test_env *te, int offset, const char *msg)
 {
 	te->flags |= TEF_ERROR;
-	internal_errorf(0, "dbteste_error: %s (offset %d)", msg, offset);
+	internal_warningf("dbteste_error: %s (offset %d)", msg, offset);
 }
