@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/expr.c,v 1.4 2006/05/10 18:54:10 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/expr.c,v 1.10 2007/03/04 03:04:25 tg Exp $");
 
 /* The order of these enums is constrained by the order of opinfo[] */
 enum token {
@@ -102,17 +102,16 @@ static const struct opinfo opinfo[] = {
 	{ "(",	 1, P_PRIMARY },
 	{ ")",	 1, P_PRIMARY },
 	{ ":",	 1, P_PRIMARY },
-	{ "",	 0, P_PRIMARY } /* end of table */
+	{ "",	 0, P_PRIMARY }
 };
 
-
 typedef struct expr_state Expr_state;
 struct expr_state {
 	const char *expression;		/* expression being evaluated */
 	const char *tokp;		/* lexical position */
-	enum token  tok;		/* token from token() */
-	int	    noassign;		/* don't do assigns (for ?:,&&,||) */
-	bool	    arith;		/* true if evaluating an $(())
+	enum token tok;			/* token from token() */
+	int noassign;			/* don't do assigns (for ?:,&&,||) */
+	bool arith;			/* true if evaluating an $(())
 					 * expression
 					 */
 	struct tbl *val;		/* value from token() */
@@ -126,12 +125,12 @@ enum error_type {
 	ET_LVALUE, ET_RDONLY, ET_STR
 };
 
-static void	   evalerr(Expr_state *, enum error_type, const char *)
-    __attribute__((__noreturn__));
+static void evalerr(Expr_state *, enum error_type, const char *)
+    __attribute__((noreturn));
 static struct tbl *evalexpr(Expr_state *, enum prec);
-static void	   token(Expr_state *);
+static void exprtoken(Expr_state *);
 static struct tbl *do_ppmm(Expr_state *, enum token, struct tbl *, bool);
-static void	   assign_check(Expr_state *, enum token, struct tbl *);
+static void assign_check(Expr_state *, enum token, struct tbl *);
 static struct tbl *tempvar(void);
 static struct tbl *intvar(Expr_state *, struct tbl *);
 
@@ -148,7 +147,7 @@ evaluate(const char *expr, long int *rval, int error_ok, bool arith)
 	v.type = 0;
 	ret = v_evaluate(&v, expr, error_ok, arith);
 	*rval = v.val.i;
-	return ret;
+	return (ret);
 }
 
 /*
@@ -178,14 +177,14 @@ v_evaluate(struct tbl *vp, const char *expr, volatile int error_ok,
 		quitenv(NULL);
 		if (i == LAEXPR) {
 			if (error_ok == KSH_RETURN_ERROR)
-				return 0;
+				return (0);
 			errorf(null);
 		}
 		unwind(i);
 		/* NOTREACHED */
 	}
 
-	token(es);
+	exprtoken(es);
 	if (es->tok == END) {
 		es->tok = LIT;
 		es->val = tempvar();
@@ -203,7 +202,7 @@ v_evaluate(struct tbl *vp, const char *expr, volatile int error_ok,
 
 	quitenv(NULL);
 
-	return 1;
+	return (1);
 }
 
 static void
@@ -274,7 +273,7 @@ evalexpr(Expr_state *es, enum prec prec)
 		op = es->tok;
 		if (op == O_BNOT || op == O_LNOT || op == O_MINUS ||
 		    op == O_PLUS) {
-			token(es);
+			exprtoken(es);
 			vl = intvar(es, evalexpr(es, P_PRIMARY));
 			if (op == O_BNOT)
 				vl->val.i = ~vl->val.i;
@@ -284,32 +283,32 @@ evalexpr(Expr_state *es, enum prec prec)
 				vl->val.i = -vl->val.i;
 			/* op == O_PLUS is a no-op */
 		} else if (op == OPEN_PAREN) {
-			token(es);
+			exprtoken(es);
 			vl = evalexpr(es, MAX_PREC);
 			if (es->tok != CLOSE_PAREN)
 				evalerr(es, ET_STR, "missing )");
-			token(es);
+			exprtoken(es);
 		} else if (op == O_PLUSPLUS || op == O_MINUSMINUS) {
-			token(es);
+			exprtoken(es);
 			vl = do_ppmm(es, op, es->val, true);
-			token(es);
+			exprtoken(es);
 		} else if (op == VAR || op == LIT) {
 			vl = es->val;
-			token(es);
+			exprtoken(es);
 		} else {
 			evalerr(es, ET_UNEXPECTED, NULL);
 			/* NOTREACHED */
 		}
 		if (es->tok == O_PLUSPLUS || es->tok == O_MINUSMINUS) {
 			vl = do_ppmm(es, es->tok, vl, false);
-			token(es);
+			exprtoken(es);
 		}
-		return vl;
+		return (vl);
 	}
 	vl = evalexpr(es, ((int) prec) - 1);
 	for (op = es->tok; IS_BINOP(op) && opinfo[(int) op].prec == prec;
 	    op = es->tok) {
-		token(es);
+		exprtoken(es);
 		vasn = vl;
 		if (op != O_ASN) /* vl may not have a value yet */
 			vl = intvar(es, vl);
@@ -411,7 +410,7 @@ evalexpr(Expr_state *es, enum prec prec)
 					es->noassign--;
 				if (es->tok != CTERN)
 					evalerr(es, ET_STR, "missing :");
-				token(es);
+				exprtoken(es);
 				if (ev)
 					es->noassign++;
 				vr = evalexpr(es, P_TERN);
@@ -437,25 +436,25 @@ evalexpr(Expr_state *es, enum prec prec)
 		} else if (op != O_TERN)
 			vl->val.i = res;
 	}
-	return vl;
+	return (vl);
 }
 
 static void
-token(Expr_state *es)
+exprtoken(Expr_state *es)
 {
 	const char *cp;
 	int c;
 	char *tvar;
 
 	/* skip white space */
-	for (cp = es->tokp; (c = *cp), isspace(c); cp++)
+	for (cp = es->tokp; (c = *cp), ksh_isspace(c); cp++)
 		;
 	es->tokp = cp;
 
 	if (c == '\0')
 		es->tok = END;
-	else if (letter(c)) {
-		for (; letnum(c); c = *cp)
+	else if (ksh_isalphx(c)) {
+		for (; ksh_isalnux(c); c = *cp)
 			cp++;
 		if (c == '[') {
 			int len;
@@ -480,8 +479,8 @@ token(Expr_state *es)
 			afree(tvar, ATEMP);
 		}
 		es->tok = VAR;
-	} else if (digit(c)) {
-		for (; c != '_' && (letnum(c) || c == '#'); c = *cp++)
+	} else if (ksh_isdigit(c)) {
+		for (; c != '_' && (ksh_isalnux(c) || c == '#'); c = *cp++)
 			;
 		tvar = str_nsave(es->tokp, --cp - es->tokp, ATEMP);
 		es->val = tempvar();
@@ -526,7 +525,7 @@ do_ppmm(Expr_state *es, enum token op, struct tbl *vasn, bool is_prefix)
 	if (!is_prefix)		/* undo the inc/dec */
 		vl->val.i = oval;
 
-	return vl;
+	return (vl);
 }
 
 static void
@@ -543,13 +542,13 @@ tempvar(void)
 {
 	struct tbl *vp;
 
-	vp = (struct tbl*) alloc(sizeof(struct tbl), ATEMP);
+	vp = (struct tbl *)alloc(sizeof (struct tbl), ATEMP);
 	vp->flag = ISSET|INTEGER;
 	vp->type = 0;
 	vp->areap = ATEMP;
 	vp->val.i = 0;
 	vp->name[0] = '\0';
-	return vp;
+	return (vp);
 }
 
 /* cast (string) variable to temporary integer variable */
@@ -561,7 +560,7 @@ intvar(Expr_state *es, struct tbl *vp)
 	/* try to avoid replacing a temp var with another temp var */
 	if (vp->name[0] == '\0' &&
 	    (vp->flag & (ISSET|INTEGER|EXPRLVALUE)) == (ISSET|INTEGER))
-		return vp;
+		return (vp);
 
 	vq = tempvar();
 	if (setint_v(vq, vp, es->arith) == NULL) {
@@ -573,5 +572,5 @@ intvar(Expr_state *es, struct tbl *vp)
 		vp->flag &= ~EXPRINEVAL;
 		es->evaling = NULL;
 	}
-	return vq;
+	return (vq);
 }
