@@ -1,14 +1,14 @@
 /*	$OpenBSD: c_ksh.c,v 1.29 2006/03/12 00:26:58 deraadt Exp $	*/
 /*	$OpenBSD: c_sh.c,v 1.35 2006/04/10 14:38:59 jaredy Exp $	*/
 /*	$OpenBSD: c_test.c,v 1.17 2005/03/30 17:16:37 deraadt Exp $	*/
-/*	$OpenBSD: c_ulimit.c,v 1.15 2006/03/12 00:26:58 deraadt Exp $	*/
+/*	$OpenBSD: c_ulimit.c,v 1.16 2006/11/20 21:53:39 miod Exp $	*/
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/funcs.c,v 1.30 2006/08/24 20:32:52 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/funcs.c,v 1.49.2.1 2007/05/13 19:29:35 tg Exp $");
 
 int
-c_cd(char **wp)
+c_cd(const char **wp)
 {
 	int optc;
 	int physical = Flag(FPHYSICAL);
@@ -21,6 +21,7 @@ c_cd(char **wp)
 	char *dir, *try, *pwd;
 	int phys_path;
 	char *cdpath;
+	bool dir_ = false;
 
 	while ((optc = ksh_getopt(wp, &builtin_opt, "LP")) != -1)
 		switch (optc) {
@@ -51,15 +52,17 @@ c_cd(char **wp)
 		}
 	} else if (!wp[1]) {
 		/* One argument: - or dir */
-		dir = wp[0];
-		if (strcmp(dir, "-") == 0) {
+		dir = str_save(wp[0], ATEMP);
+		if (ksh_isdash(dir)) {
+			afree(dir, ATEMP);
 			dir = str_val(oldpwd_s);
 			if (dir == null) {
 				bi_errorf("no OLDPWD");
 				return 1;
 			}
 			printpath++;
-		}
+		} else
+			dir_ = true;
 	} else if (!wp[2]) {
 		/* Two arguments - substitute arg1 in PWD for arg2 */
 		int ilen, olen, nlen, elen;
@@ -83,6 +86,7 @@ c_cd(char **wp)
 		nlen = strlen(wp[1]);
 		elen = strlen(current_wd + ilen + olen) + 1;
 		dir = alloc(ilen + nlen + elen, ATEMP);
+		dir_ = true;
 		memcpy(dir, current_wd, ilen);
 		memcpy(dir + ilen, wp[1], nlen);
 		memcpy(dir + ilen + nlen, current_wd + ilen + olen, elen);
@@ -114,6 +118,7 @@ c_cd(char **wp)
 			bi_errorf("%s: bad directory", dir);
 		else
 			bi_errorf("%s - %s", try, strerror(errno));
+		afreechv(dir_, dir);
 		return 1;
 	}
 
@@ -147,11 +152,12 @@ c_cd(char **wp)
 	if (printpath || cdnode)
 		shprintf("%s\n", pwd);
 
+	afreechv(dir_, dir);
 	return 0;
 }
 
 int
-c_pwd(char **wp)
+c_pwd(const char **wp)
 {
 	int optc;
 	int physical = Flag(FPHYSICAL);
@@ -176,22 +182,18 @@ c_pwd(char **wp)
 	}
 	p = current_wd[0] ? (physical ? get_phys_path(current_wd) : current_wd) :
 	    NULL;
-	if (p && eaccess(p, R_OK) < 0)
+	if (p && access(p, R_OK) < 0)
 		p = NULL;
-	if (!p) {
-		p = ksh_get_wd(NULL, 0);
-		if (!p) {
-			bi_errorf("can't get current directory - %s",
-			    strerror(errno));
-			return 1;
-		}
+	if (!p && !(p = ksh_get_wd(NULL))) {
+		bi_errorf("can't get current directory - %s", strerror(errno));
+		return 1;
 	}
 	shprintf("%s\n", p);
 	return 0;
 }
 
 int
-c_print(char **wp)
+c_print(const char **wp)
 {
 #define PO_NL		BIT(0)	/* print newline */
 #define PO_EXPAND	BIT(1)	/* expand backslash sequences */
@@ -200,7 +202,7 @@ c_print(char **wp)
 #define PO_COPROC	BIT(4)	/* printing to coprocess: block SIGPIPE */
 	int fd = 1;
 	int flags = PO_EXPAND|PO_NL;
-	char *s;
+	const char *s;
 	const char *emsg;
 	XString xs;
 	char *xp;
@@ -210,10 +212,10 @@ c_print(char **wp)
 
 		/* A compromise between sysV and BSD echo commands:
 		 * escape sequences are enabled by default, and
-		 * -n, -e and -E are recognized if they appear
+		 * -n, -e and -E are recognised if they appear
 		 * in arguments with no illegal options (ie, echo -nq
 		 * will print -nq).
-		 * Different from sysV echo since options are recognized,
+		 * Different from sysV echo since options are recognised,
 		 * different from BSD echo since escape sequences are enabled
 		 * by default.
 		 */
@@ -278,7 +280,7 @@ c_print(char **wp)
 		if (!(builtin_opt.info & GI_MINUSMINUS)) {
 			/* treat a lone - like -- */
 			if (wp[builtin_opt.optind] &&
-			    strcmp(wp[builtin_opt.optind], "-") == 0)
+			    ksh_isdash(wp[builtin_opt.optind]))
 				builtin_opt.optind++;
 		} else if (flags & PO_PMINUSMINUS)
 			builtin_opt.optind--;
@@ -379,10 +381,10 @@ c_print(char **wp)
 }
 
 int
-c_whence(char **wp)
+c_whence(const char **wp)
 {
 	struct tbl *tp;
-	char *id;
+	const char *id;
 	int pflag = 0, vflag = 0, Vflag = 0;
 	int ret = 0;
 	int optc;
@@ -499,7 +501,7 @@ c_whence(char **wp)
 
 /* Deal with command -vV - command -p dealt with in comexec() */
 int
-c_command(char **wp)
+c_command(const char **wp)
 {
 	/* Let c_whence do the work.  Note that c_command() must be
 	 * a distinct function from c_whence() (tested in comexec()).
@@ -509,14 +511,14 @@ c_command(char **wp)
 
 /* typeset, export, and readonly */
 int
-c_typeset(char **wp)
+c_typeset(const char **wp)
 {
 	struct block *l = e->loc;
 	struct tbl *vp, **p;
 	Tflag fset = 0, fclr = 0;
 	int thing = 0, func = 0, localv = 0;
 	const char *opts = "L#R#UZ#fi#lprtux";	/* see comment below */
-	char *fieldstr, *basestr;
+	const char *fieldstr, *basestr;
 	int field, base;
 	int optc;
 	Tflag flag;
@@ -587,7 +589,7 @@ c_typeset(char **wp)
 			 * here for compatibility with ksh93.
 			 */
 			pflag = 1;
-			break;
+			continue;
 		case 'r':
 			flag = RDONLY;
 			break;
@@ -772,9 +774,7 @@ c_typeset(char **wp)
 						break;
 					} else {
 						if (pflag)
-							shprintf("%s ",
-							    (flag & EXPORT) ?
-							    "export" : "readonly");
+							shprintf("typeset ");
 						if ((vp->flag&ARRAY) && any_set)
 							shprintf("%s[%d]",
 							    vp->name, vp->index);
@@ -808,7 +808,7 @@ c_typeset(char **wp)
 }
 
 int
-c_alias(char **wp)
+c_alias(const char **wp)
 {
 	struct table *t = &aliases;
 	int rv = 0, rflag = 0, tflag, Uflag = 0, pflag = 0;
@@ -821,7 +821,11 @@ c_alias(char **wp)
 		prefix = builtin_opt.info & GI_PLUS ? '+' : '-';
 		switch (optc) {
 		case 'd':
+#ifdef MKSH_SMALL
+			return (0);
+#else
 			t = &homedirs;
+#endif
 			break;
 		case 'p':
 			pflag = 1;
@@ -858,7 +862,7 @@ c_alias(char **wp)
 
 	/* "hash -r" means reset all the tracked aliases.. */
 	if (rflag) {
-		static const char *const args[] = {
+		static const char *args[] = {
 			"unalias", "-ta", NULL
 		};
 
@@ -868,7 +872,7 @@ c_alias(char **wp)
 			return 1;
 		}
 		ksh_getopt_reset(&builtin_opt, GF_ERROR);
-		return c_unalias((char **) args);
+		return c_unalias(args);
 	}
 
 	if (*wp == NULL) {
@@ -888,14 +892,15 @@ c_alias(char **wp)
 	}
 
 	for (; *wp != NULL; wp++) {
-		char *alias = *wp;
-		char *val = strchr(alias, '=');
-		char *newval;
+		const char *alias = *wp;
+		char *xalias = NULL;
+		const char *val;
+		const char *newval;
 		struct tbl *ap;
 		int h;
 
-		if (val)
-			alias = str_nsave(alias, val++ - alias, ATEMP);
+		if ((val = cstrchr(alias, '=')))
+			alias = xalias = str_nsave(alias, val++ - alias, ATEMP);
 		h = hash(alias);
 		if (val == NULL && !tflag && !xflag) {
 			ap = ktsearch(t, alias, h);
@@ -936,15 +941,14 @@ c_alias(char **wp)
 			ap->flag &= ~xflag;
 		else
 			ap->flag |= xflag;
-		if (val)
-			afree(alias, ATEMP);
+		afreechk(xalias);
 	}
 
 	return rv;
 }
 
 int
-c_unalias(char **wp)
+c_unalias(const char **wp)
 {
 	struct table *t = &aliases;
 	struct tbl *ap;
@@ -957,7 +961,11 @@ c_unalias(char **wp)
 			all = 1;
 			break;
 		case 'd':
+#ifdef MKSH_SMALL
+			return (0);
+#else
 			t = &homedirs;
+#endif
 			break;
 		case 't':
 			t = &taliases;
@@ -996,7 +1004,7 @@ c_unalias(char **wp)
 }
 
 int
-c_let(char **wp)
+c_let(const char **wp)
 {
 	int rv = 1;
 	long val;
@@ -1014,7 +1022,7 @@ c_let(char **wp)
 }
 
 int
-c_jobs(char **wp)
+c_jobs(const char **wp)
 {
 	int optc;
 	int flag = 0;
@@ -1051,7 +1059,7 @@ c_jobs(char **wp)
 }
 
 int
-c_fgbg(char **wp)
+c_fgbg(const char **wp)
 {
 	int bg = strcmp(*wp, "bg") == 0;
 	int rv = 0;
@@ -1075,40 +1083,34 @@ struct kill_info {
 	int num_width;
 	int name_width;
 };
-static char *kill_fmt_entry(void *arg, int i, char *buf, int buflen);
+static char *kill_fmt_entry(const void *, int, char *, int);
 
 /* format a single kill item */
 static char *
-kill_fmt_entry(void *arg, int i, char *buf, int buflen)
+kill_fmt_entry(const void *arg, int i, char *buf, int buflen)
 {
-	struct kill_info *ki = (struct kill_info *) arg;
+	const struct kill_info *ki = (const struct kill_info *)arg;
 
 	i++;
-	if (sigtraps[i].name)
-		shf_snprintf(buf, buflen, "%*d %*s %s",
-		    ki->num_width, i,
-		    ki->name_width, sigtraps[i].name,
-		    sigtraps[i].mess);
-	else
-		shf_snprintf(buf, buflen, "%*d %*d %s",
-		    ki->num_width, i,
-		    ki->name_width, sigtraps[i].signal,
-		    sigtraps[i].mess);
+	shf_snprintf(buf, buflen, "%*d %*s %s",
+	    ki->num_width, i,
+	    ki->name_width, sigtraps[i].name,
+	    sigtraps[i].mess);
 	return buf;
 }
 
 
 int
-c_kill(char **wp)
+c_kill(const char **wp)
 {
 	Trap *t = NULL;
-	char *p;
+	const char *p;
 	int lflag = 0;
 	int i, n, rv, sig;
 
 	/* assume old style options if -digits or -UPPERCASE */
-	if ((p = wp[1]) && *p == '-' && (digit(p[1]) ||
-	    isupper((unsigned char)p[1]))) {
+	if ((p = wp[1]) && *p == '-' && (ksh_isdigit(p[1]) ||
+	    ksh_isupper(p[1]))) {
 		if (!(t = gettrap(p + 1, true))) {
 			bi_errorf("bad signal '%s'", p + 1);
 			return 1;
@@ -1135,9 +1137,11 @@ c_kill(char **wp)
 		i = builtin_opt.optind;
 	}
 	if ((lflag && t) || (!wp[i] && !lflag)) {
+#ifndef MKSH_SMALL
 		shf_fprintf(shl_out,
 		    "Usage: kill [ -s signame | -signum | -signame ] {pid|job}...\n"
 		    "       kill -l [exit_status]\n");
+#endif
 		bi_errorf(null);
 		return 1;
 	}
@@ -1149,7 +1153,7 @@ c_kill(char **wp)
 					return 1;
 				if (n > 128 && n < 128 + NSIG)
 					n -= 128;
-				if (n > 0 && n < NSIG && sigtraps[n].name)
+				if (n > 0 && n < NSIG)
 					shprintf("%s\n", sigtraps[n].name);
 				else
 					shprintf("%d\n", n);
@@ -1163,9 +1167,7 @@ c_kill(char **wp)
 				ki.num_width++;
 			ki.name_width = mess_width = 0;
 			for (j = 0; j < NSIG; j++) {
-				w = sigtraps[j].name ?
-				    (int)strlen(sigtraps[j].name) :
-				    ki.num_width;
+				w = strlen(sigtraps[j].name);
 				if (w > ki.name_width)
 					ki.name_width = w;
 				w = strlen(sigtraps[j].mess);
@@ -1174,7 +1176,7 @@ c_kill(char **wp)
 			}
 
 			print_columns(shl_stdout, NSIG - 1,
-			    kill_fmt_entry, (void *) &ki,
+			    kill_fmt_entry, (void *)&ki,
 			    ki.num_width + ki.name_width + mess_width + 3, 1);
 		}
 		return 0;
@@ -1212,7 +1214,7 @@ getopts_reset(int val)
 }
 
 int
-c_getopts(char **wp)
+c_getopts(const char **wp)
 {
 	int	argc;
 	const char *opts;
@@ -1243,7 +1245,7 @@ c_getopts(char **wp)
 	}
 
 	if (e->loc->next == NULL) {
-		internal_errorf(0, "c_getopts: no argv");
+		internal_warningf("c_getopts: no argv");
 		return 1;
 	}
 	/* Which arguments are we parsing... */
@@ -1309,10 +1311,11 @@ c_getopts(char **wp)
 }
 
 int
-c_bind(char **wp)
+c_bind(const char **wp)
 {
 	int optc, rv = 0, macro = 0, list = 0;
-	char *cp;
+	const char *cp;
+	char *up;
 
 	while ((optc = ksh_getopt(wp, &builtin_opt, "lm")) != -1)
 		switch (optc) {
@@ -1331,11 +1334,15 @@ c_bind(char **wp)
 		rv = x_bind((char*)NULL, (char*)NULL, 0, list);
 
 	for (; *wp != NULL; wp++) {
-		cp = strchr(*wp, '=');
-		if (cp != NULL)
-			*cp++ = '\0';
-		if (x_bind(*wp, cp, macro, 0))
+		if ((cp = cstrchr(*wp, '=')) == NULL)
+			up = NULL;
+		else {
+			up = str_save(*wp, ATEMP);
+			up[cp++ - *wp] = '\0';
+		}
+		if (x_bind(up ? up : *wp, cp, macro, 0))
 			rv = 1;
+		afreechk(up);
 	}
 
 	return rv;
@@ -1374,18 +1381,18 @@ static void p_time(struct shf *, int, struct timeval *, int,
 
 /* :, false and true */
 int
-c_label(char **wp)
+c_label(const char **wp)
 {
 	return wp[0][0] == 'f' ? 1 : 0;
 }
 
 int
-c_shift(char **wp)
+c_shift(const char **wp)
 {
 	struct block *l = e->loc;
 	int n;
 	long val;
-	char *arg;
+	const char *arg;
 
 	if (ksh_getopt(wp, &builtin_opt, null) == '?')
 		return 1;
@@ -1411,10 +1418,10 @@ c_shift(char **wp)
 }
 
 int
-c_umask(char **wp)
+c_umask(const char **wp)
 {
 	int i;
-	char *cp;
+	const char *cp;
 	int symbolic = 0;
 	mode_t old_umask;
 	int optc;
@@ -1432,27 +1439,27 @@ c_umask(char **wp)
 		old_umask = umask(0);
 		umask(old_umask);
 		if (symbolic) {
-			char buf[18];
+			char buf[18], *p;
 			int j;
 
 			old_umask = ~old_umask;
-			cp = buf;
+			p = buf;
 			for (i = 0; i < 3; i++) {
-				*cp++ = "ugo"[i];
-				*cp++ = '=';
+				*p++ = "ugo"[i];
+				*p++ = '=';
 				for (j = 0; j < 3; j++)
 					if (old_umask & (1 << (8 - (3*i + j))))
-						*cp++ = "rwx"[j];
-				*cp++ = ',';
+						*p++ = "rwx"[j];
+				*p++ = ',';
 			}
-			cp[-1] = '\0';
+			p[-1] = '\0';
 			shprintf("%s\n", buf);
 		} else
 			shprintf("%#3.3o\n", (unsigned) old_umask);
 	} else {
 		mode_t new_umask;
 
-		if (digit(*cp)) {
+		if (ksh_isdigit(*cp)) {
 			for (new_umask = 0; *cp >= '0' && *cp <= '7'; cp++)
 				new_umask = new_umask * 8 + (*cp - '0');
 			if (*cp) {
@@ -1470,7 +1477,7 @@ c_umask(char **wp)
 			new_umask = old_umask;
 			positions = 0;
 			while (*cp) {
-				while (*cp && strchr("augo", *cp))
+				while (*cp && vstrchr("augo", *cp))
 					switch (*cp++) {
 					case 'a':
 						positions |= 0111;
@@ -1487,11 +1494,11 @@ c_umask(char **wp)
 					}
 				if (!positions)
 					positions = 0111; /* default is a */
-				if (!strchr("=+-", op = *cp))
+				if (!vstrchr("=+-", op = *cp))
 					break;
 				cp++;
 				new_val = 0;
-				while (*cp && strchr("rwxugoXs", *cp))
+				while (*cp && vstrchr("rwxugoXs", *cp))
 					switch (*cp++) {
 					case 'r': new_val |= 04; break;
 					case 'w': new_val |= 02; break;
@@ -1523,7 +1530,7 @@ c_umask(char **wp)
 				if (*cp == ',') {
 					positions = 0;
 					cp++;
-				} else if (!strchr("=+-", *cp))
+				} else if (!vstrchr("=+-", *cp))
 					break;
 			}
 			if (*cp) {
@@ -1538,10 +1545,11 @@ c_umask(char **wp)
 }
 
 int
-c_dot(char **wp)
+c_dot(const char **wp)
 {
-	char *file, *cp;
-	char **argv;
+	const char *file;
+	const char *cp;
+	const char **argv;
 	int argc;
 	int i;
 	int err;
@@ -1576,7 +1584,7 @@ c_dot(char **wp)
 }
 
 int
-c_wait(char **wp)
+c_wait(const char **wp)
 {
 	int rv = 0;
 	int sig;
@@ -1598,20 +1606,21 @@ c_wait(char **wp)
 }
 
 int
-c_read(char **wp)
+c_read(const char **wp)
 {
 	int c = 0;
 	int expande = 1, historyr = 0;
 	int expanding;
 	int ecode = 0;
-	char *cp;
+	const char *cp;
+	char *ccp;
 	int fd = 0;
 	struct shf *shf;
 	int optc;
 	const char *emsg;
 	XString cs, xs = { NULL, NULL, 0, NULL};
 	struct tbl *vp;
-	char *xp = NULL;
+	char *xp = NULL, *wpalloc = NULL;
 	static char REPLY[] = "REPLY";
 
 	while ((optc = ksh_getopt(wp, &builtin_opt, "prsu,")) != -1)
@@ -1649,8 +1658,10 @@ c_read(char **wp)
 	 */
 	shf = shf_reopen(fd, SHF_RD | SHF_INTERRUPT | can_seek(fd), shl_spare);
 
-	if ((cp = strchr(*wp, '?')) != NULL) {
-		*cp = 0;
+	if ((cp = cstrchr(*wp, '?')) != NULL) {
+		wpalloc = str_save(*wp, ATEMP);
+		wpalloc[cp - *wp] = '\0';
+		*wp = wpalloc;
 		if (isatty(fd)) {
 			/* at&t ksh says it prints prompt on fd if it's open
 			 * for writing and is a tty, but it doesn't do it
@@ -1675,9 +1686,9 @@ c_read(char **wp)
 	if (historyr)
 		Xinit(xs, xp, 128, ATEMP);
 	expanding = 0;
-	Xinit(cs, cp, 128, ATEMP);
+	Xinit(cs, ccp, 128, ATEMP);
 	for (; *wp != NULL; wp++) {
-		for (cp = Xstring(cs, cp); ; ) {
+		for (ccp = Xstring(cs, ccp); ; ) {
 			if (c == '\n' || c == EOF)
 				break;
 			while (1) {
@@ -1704,7 +1715,7 @@ c_read(char **wp)
 				Xcheck(xs, xp);
 				Xput(xs, xp, c);
 			}
-			Xcheck(cs, cp);
+			Xcheck(cs, ccp);
 			if (expanding) {
 				expanding = 0;
 				if (c == '\n') {
@@ -1717,7 +1728,7 @@ c_read(char **wp)
 						pprompt(prompt, 0);
 					}
 				} else if (c != EOF)
-					Xput(cs, cp, c);
+					Xput(cs, ccp, c);
 				continue;
 			}
 			if (expande && c == '\\') {
@@ -1727,31 +1738,33 @@ c_read(char **wp)
 			if (c == '\n' || c == EOF)
 				break;
 			if (ctype(c, C_IFS)) {
-				if (Xlength(cs, cp) == 0 && ctype(c, C_IFSWS))
+				if (Xlength(cs, ccp) == 0 && ctype(c, C_IFSWS))
 					continue;
 				if (wp[1])
 					break;
 			}
-			Xput(cs, cp, c);
+			Xput(cs, ccp, c);
 		}
 		/* strip trailing IFS white space from last variable */
 		if (!wp[1])
-			while (Xlength(cs, cp) && ctype(cp[-1], C_IFS) &&
-			    ctype(cp[-1], C_IFSWS))
-				cp--;
-		Xput(cs, cp, '\0');
+			while (Xlength(cs, ccp) && ctype(ccp[-1], C_IFS) &&
+			    ctype(ccp[-1], C_IFSWS))
+				ccp--;
+		Xput(cs, ccp, '\0');
 		vp = global(*wp);
 		/* Must be done before setting export. */
 		if (vp->flag & RDONLY) {
 			shf_flush(shf);
 			bi_errorf("%s is read only", *wp);
+			afreechk(wpalloc);
 			return 1;
 		}
 		if (Flag(FEXPORT))
 			typeset(*wp, EXPORT, 0, 0, 0);
-		if (!setstr(vp, Xstring(cs, cp), KSH_RETURN_ERROR)) {
-		    shf_flush(shf);
-		    return 1;
+		if (!setstr(vp, Xstring(cs, ccp), KSH_RETURN_ERROR)) {
+			shf_flush(shf);
+			afreechk(wpalloc);
+			return 1;
 		}
 	}
 
@@ -1769,11 +1782,12 @@ c_read(char **wp)
 	if (c == EOF && !ecode)
 		coproc_read_close(fd);
 
+	afreechk(wpalloc);
 	return ecode ? ecode : c == EOF;
 }
 
 int
-c_eval(char **wp)
+c_eval(const char **wp)
 {
 	struct source *s, *saves = source;
 	int savef, rv;
@@ -1818,10 +1832,10 @@ c_eval(char **wp)
 }
 
 int
-c_trap(char **wp)
+c_trap(const char **wp)
 {
 	int i;
-	char *s;
+	const char *s;
 	Trap *p;
 
 	if (ksh_getopt(wp, &builtin_opt, null) == '?')
@@ -1865,11 +1879,11 @@ c_trap(char **wp)
 }
 
 int
-c_exitreturn(char **wp)
+c_exitreturn(const char **wp)
 {
 	int how = LEXIT;
 	int n;
-	char *arg;
+	const char *arg;
 
 	if (ksh_getopt(wp, &builtin_opt, null) == '?')
 		return 1;
@@ -1903,15 +1917,14 @@ c_exitreturn(char **wp)
 	quitenv(NULL);	/* get rid of any i/o redirections */
 	unwind(how);
 	/* NOTREACHED */
-	return 0;
 }
 
 int
-c_brkcont(char **wp)
+c_brkcont(const char **wp)
 {
 	int n, quit;
 	struct env *ep, *last_ep = NULL;
-	char *arg;
+	const char *arg;
 
 	if (ksh_getopt(wp, &builtin_opt, null) == '?')
 		return 1;
@@ -1950,7 +1963,8 @@ c_brkcont(char **wp)
 		 * shall be used.  Doesn't say to print an error but we
 		 * do anyway 'cause the user messed up.
 		 */
-		last_ep->flags &= ~EF_BRKCONT_PASS;
+		if (last_ep)
+			last_ep->flags &= ~EF_BRKCONT_PASS;
 		warningf(true, "%s: can only %s %d level(s)",
 		    wp[0], wp[0], n - quit);
 	}
@@ -1960,15 +1974,15 @@ c_brkcont(char **wp)
 }
 
 int
-c_set(char **wp)
+c_set(const char **wp)
 {
 	int argi, setargs;
 	struct block *l = e->loc;
-	char **owp = wp;
+	const char **owp = wp;
 
 	if (wp[1] == NULL) {
-		static const char *const args [] = { "set", "-", NULL };
-		return c_typeset((char **) args);
+		static const char *args [] = { "set", "-", NULL };
+		return c_typeset(args);
 	}
 
 	argi = parse_args(wp, OF_SET, &setargs);
@@ -1981,7 +1995,8 @@ c_set(char **wp)
 		while (*++wp != NULL)
 			*wp = str_save(*wp, &l->area);
 		l->argc = wp - owp - 1;
-		l->argv = (char **) alloc(sizeofN(char *, l->argc+2), &l->area);
+		l->argv = (const char **) alloc(sizeofN(char *, l->argc+2),
+		     &l->area);
 		for (wp = l->argv; (*wp++ = *owp++) != NULL; )
 			;
 	}
@@ -1995,9 +2010,9 @@ c_set(char **wp)
 }
 
 int
-c_unset(char **wp)
+c_unset(const char **wp)
 {
-	char *id;
+	const char *id;
 	int optc, unset_var = 1;
 	int ret = 0;
 
@@ -2023,7 +2038,7 @@ c_unset(char **wp)
 				bi_errorf("%s is read only", vp->name);
 				return 1;
 			}
-			unset(vp, strchr(id, '[') ? 1 : 0);
+			unset(vp, vstrchr(id, '[') ? 1 : 0);
 		} else {		/* unset function */
 			if (define(id, (struct op *) NULL))
 				ret = 1;
@@ -2045,7 +2060,7 @@ p_time(struct shf *shf, int posix, struct timeval *tv, int width,
 }
 
 int
-c_times(char **wp __attribute__((unused)))
+c_times(const char **wp __unused)
 {
 	struct rusage usage;
 
@@ -2080,7 +2095,6 @@ timex(struct op *t, int f)
 	struct rusage ru0, ru1, cru0, cru1;
 	struct timeval usrtime, systime, tv0, tv1;
 	int tf = 0;
-	extern struct timeval j_usrtime, j_systime; /* computed by j_wait */
 	char opts[1];
 
 	gettimeofday(&tv0, NULL);
@@ -2150,7 +2164,7 @@ timex_hook(struct op *t, char **volatile *app)
 
 	ksh_getopt_reset(&opt, 0);
 	opt.optind = 0;	/* start at the start */
-	while ((optc = ksh_getopt(wp, &opt, ":p")) != -1)
+	while ((optc = ksh_getopt((const char **)wp, &opt, ":p")) != -1)
 		switch (optc) {
 		case 'p':
 			t->str[0] |= TF_POSIX;
@@ -2175,7 +2189,7 @@ timex_hook(struct op *t, char **volatile *app)
 
 /* exec with no args - args case is taken care of in comexec() */
 int
-c_exec(char **wp __attribute__((unused)))
+c_exec(const char **wp __unused)
 {
 	int i;
 
@@ -2193,12 +2207,13 @@ c_exec(char **wp __attribute__((unused)))
 	return 0;
 }
 
+#if !defined(MKSH_SMALL) || defined(MKSH_NEED_MKNOD)
 static int
-c_mknod(char **wp)
+c_mknod(const char **wp)
 {
 	int argc, optc, rv = 0;
 	bool ismkfifo = false;
-	char **argv;
+	const char **argv;
 	void *set = NULL;
 	mode_t mode = 0, oldmode = 0;
 
@@ -2276,10 +2291,11 @@ c_mknod(char **wp)
 	bi_errorf("usage: mknod [-m mode] name p");
 	return (1);
 }
+#endif
 
 /* dummy function, special case in comexec() */
 int
-c_builtin(char **wp __attribute__((unused)))
+c_builtin(const char **wp __unused)
 {
 	return 0;
 }
@@ -2312,7 +2328,9 @@ const struct builtin shbuiltins [] = {
 	{"ulimit", c_ulimit},
 	{"+umask", c_umask},
 	{"*=unset", c_unset},
+#if !defined(MKSH_SMALL) || defined(MKSH_NEED_MKNOD)
 	{"mknod", c_mknod},
+#endif
 	{NULL, NULL}
 };
 
@@ -2395,12 +2413,10 @@ static int	test_nexpr(Test_env *, int);
 static int	test_primary(Test_env *, int);
 static int	ptest_isa(Test_env *, Test_meta);
 static const char *ptest_getopnd(Test_env *, Test_op, int);
-static int	ptest_eval(Test_env *, Test_op, const char *,
-		    const char *, int);
 static void	ptest_error(Test_env *, int, const char *);
 
 int
-c_test(char **wp)
+c_test(const char **wp)
 {
 	int argc;
 	int res;
@@ -2409,7 +2425,7 @@ c_test(char **wp)
 	te.flags = 0;
 	te.isa = ptest_isa;
 	te.getopnd = ptest_getopnd;
-	te.eval = ptest_eval;
+	te.eval = test_eval;
 	te.error = ptest_error;
 
 	for (argc = 0; wp[argc]; argc++)
@@ -2431,7 +2447,7 @@ c_test(char **wp)
 	 * our parser does the right thing for the omitted steps.
 	 */
 	if (argc <= 5) {
-		char **owp = wp;
+		const char **owp = wp;
 		int invert = 0;
 		Test_op	op;
 		const char *opnd1, *opnd2;
@@ -2499,14 +2515,14 @@ int
 test_eval(Test_env *te, Test_op op, const char *opnd1, const char *opnd2,
     int do_eval)
 {
-	int res;
-	int not;
+	int i;
+	size_t k;
 	struct stat b1, b2;
 
 	if (!do_eval)
 		return 0;
 
-	switch ((int) op) {
+	switch ((int)op) {
 	/*
 	 * Unary Operators
 	 */
@@ -2515,16 +2531,16 @@ test_eval(Test_env *te, Test_op op, const char *opnd1, const char *opnd2,
 	case TO_STZER: /* -z */
 		return *opnd1 == '\0';
 	case TO_OPTION: /* -o */
-		if ((not = *opnd1 == '!'))
+		if ((i = *opnd1 == '!'))
 			opnd1++;
-		if ((res = option(opnd1)) < 0)
-			res = 0;
+		if ((k = option(opnd1)) == (size_t)-1)
+			k = 0;
 		else {
-			res = Flag(res);
-			if (not)
-				res = !res;
+			k = Flag(k);
+			if (i)
+				k = !k;
 		}
-		return res;
+		return k;
 	case TO_FILRD: /* -r */
 		return test_eaccess(opnd1, R_OK) == 0;
 	case TO_FILWR: /* -w */
@@ -2566,12 +2582,12 @@ test_eval(Test_env *te, Test_op op, const char *opnd1, const char *opnd2,
 	case TO_FILGZ: /* -s */
 		return stat(opnd1, &b1) == 0 && b1.st_size > 0L;
 	case TO_FILTT: /* -t */
-		if (opnd1 && !bi_getn(opnd1, &res)) {
+		if (opnd1 && !bi_getn(opnd1, &i)) {
 			te->flags |= TEF_ERROR;
-			res = 0;
+			i = 0;
 		} else
-			res = isatty(opnd1 ? res : 0);
-		return res;
+			i = isatty(opnd1 ? i : 0);
+		return (i);
 	case TO_FILUID: /* -O */
 		return stat(opnd1, &b1) == 0 && b1.st_uid == ksheuid;
 	case TO_FILGID: /* -G */
@@ -2653,7 +2669,7 @@ test_eval(Test_env *te, Test_op op, const char *opnd1, const char *opnd2,
 static int
 test_eaccess(const char *pathl, int mode)
 {
-	int res = eaccess(pathl, mode);
+	int res = access(pathl, mode);
 
 	if (res == 0 && ksheuid == 0 && (mode & X_OK)) {
 		struct stat statb;
@@ -2802,20 +2818,13 @@ ptest_isa(Test_env *te, Test_meta meta)
 }
 
 static const char *
-ptest_getopnd(Test_env *te, Test_op op, int do_eval __attribute__((unused)))
+ptest_getopnd(Test_env *te, Test_op op, int do_eval __unused)
 {
 	if (te->pos.wp >= te->wp_end)
 		return op == TO_FILTT ? "1" : NULL;
 	return *te->pos.wp++;
 }
 
-static int
-ptest_eval(Test_env *te, Test_op op, const char *opnd1, const char *opnd2,
-    int do_eval)
-{
-	return test_eval(te, op, opnd1, opnd2, do_eval);
-}
-
 static void
 ptest_error(Test_env *te, int offset, const char *msg)
 {
@@ -2833,7 +2842,7 @@ ptest_error(Test_env *te, int offset, const char *msg)
 #define HARD	0x2
 
 int
-c_ulimit(char **wp)
+c_ulimit(const char **wp)
 {
 	static const struct limits {
 		const char	*name;
@@ -2935,7 +2944,7 @@ c_ulimit(char **wp)
 	for (l = limits; l->name && l->option != what; l++)
 		;
 	if (!l->name) {
-		internal_errorf(0, "ulimit: %c", what);
+		internal_warningf("ulimit: %c", what);
 		return 1;
 	}
 
@@ -2960,11 +2969,11 @@ c_ulimit(char **wp)
 			 * add parameter to evaluate() to control
 			 * if unset params are 0 or an error.
 			 */
-			if (!rval && !digit(wp[0][0])) {
+			if (!rval && !ksh_isdigit(wp[0][0])) {
 				bi_errorf("invalid limit: %s", wp[0]);
 				return 1;
 			}
-			val = rval * l->factor;
+			val = (rlim_t)rval * l->factor;
 		}
 	}
 	if (all) {
@@ -2986,7 +2995,7 @@ c_ulimit(char **wp)
 					val = limit.rlim_max;
 			}
 			shprintf("%-20s ", l->name);
-			if (val == RLIM_INFINITY)
+			if (val == (rlim_t)RLIM_INFINITY)
 				shprintf("unlimited\n");
 			else {
 				val /= l->factor;
@@ -3018,7 +3027,7 @@ c_ulimit(char **wp)
 		}
 	}
 	if (!set) {
-		if (val == RLIM_INFINITY)
+		if (val == (rlim_t)RLIM_INFINITY)
 			shprintf("unlimited\n");
 		else {
 			val /= l->factor;
