@@ -3,31 +3,12 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.25 2006/08/24 18:57:30 tg Exp $");
-
-#if !defined(__sun__)
-#define DO_HISTORY
-#endif
-
-#ifndef mksh_siglist
-#if defined(BSD) || defined(__APPLE__)
-#define	mksh_signame(x)	sys_signame[(x)]
-#define	mksh_siglist(x)	sys_siglist[(x)]
-#elif defined(__INTERIX)
-#define	mksh_signame(x)	_sys_signame[(x)]
-#define	mksh_siglist(x) _sys_siglist[(x)]
-#elif defined(__gnu_linux__) || defined(__sun__) || defined(__CYGWIN__)
-#define	NEED_MKSH_SIGNAME	/* sync the list above with Build.sh */
-#define	mksh_siglist(x)	strsignal(x)
-#else
-# error "Define sys_sig{name,list} for this platform!"
-#endif
-#endif /* ndef mksh_siglist */
+__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.45.2.1 2007/05/13 19:29:36 tg Exp $");
 
 Trap sigtraps[NSIG + 1];
 static struct sigaction Sigact_ign, Sigact_trap;
 
-#ifdef DO_HISTORY
+#if HAVE_PERSISTENT_HISTORY
 static int hist_count_lines(unsigned char *, int);
 static int hist_shrink(unsigned char *, int);
 static unsigned char *hist_skip_back(unsigned char *,int *,int);
@@ -37,29 +18,32 @@ static void writehistfile(int, char *);
 static int sprinkle(int);
 #endif
 
-static int	hist_execute(char *);
-static int	hist_replace(char **, const char *, const char *, int);
-static char   **hist_get(const char *, int, int);
-static char   **hist_get_oldest(void);
-static void	histbackup(void);
+static int hist_execute(char *);
+static int hist_replace(char **, const char *, const char *, int);
+static char **hist_get(const char *, int, int);
+static char **hist_get_oldest(void);
+static void histbackup(void);
 
-static char   **current;	/* current position in history[] */
-static char    *hname;		/* current name of history file */
-static int	hstarted;	/* set after hist_init() called */
-static Source	*hist_source;
+static char **current;		/* current position in history[] */
+static int hstarted;		/* set after hist_init() called */
+static Source *hist_source;
 
-static int	histfd;
-static int	hsize;
+#if HAVE_PERSISTENT_HISTORY
+static char *hname;		/* current name of history file */
+static int histfd;
+static int hsize;
+#endif
 
 int
-c_fc(char **wp)
+c_fc(const char **wp)
 {
 	struct shf *shf;
 	struct temp *tf = NULL;
-	char *p, *editor = NULL;
+	const char *p;
+	char *editor = NULL;
 	int gflag = 0, lflag = 0, nflag = 0, sflag = 0, rflag = 0;
 	int optc;
-	char *first = NULL, *last = NULL;
+	const char *first = NULL, *last = NULL;
 	char **hfirst, **hlast, **hp;
 
 	if (!Flag(FTALKING_I)) {
@@ -72,12 +56,12 @@ c_fc(char **wp)
 		switch (optc) {
 		case 'e':
 			p = builtin_opt.optarg;
-			if (strcmp(p, "-") == 0)
+			if (ksh_isdash(p))
 				sflag++;
 			else {
-				size_t len = strlen(p) + 4;
-				editor = str_nsave(p, len, ATEMP);
-				strlcat(editor, " $_", len);
+				size_t len = strlen(p);
+				editor = str_nsave(p, len + 4, ATEMP);
+				memcpy(editor + len, " $_", 4);
 			}
 			break;
 		case 'g': /* non-at&t ksh */
@@ -124,11 +108,10 @@ c_fc(char **wp)
 		}
 
 		/* Check for pattern replacement argument */
-		if (*wp && **wp && (p = strchr(*wp + 1, '='))) {
+		if (*wp && **wp && (p = cstrchr(*wp + 1, '='))) {
 			pat = str_save(*wp, ATEMP);
-			p = pat + (p - *wp);
-			*p++ = '\0';
-			rep = p;
+			rep = pat + (p - *wp);
+			*rep++ = '\0';
 			wp++;
 		}
 		/* Check for search prefix */
@@ -198,7 +181,8 @@ c_fc(char **wp)
 			    hist_source->line - (int) (histptr - hp));
 			/* print multi-line commands correctly */
 			for (s = *hp; (t = strchr(s, '\n')); s = t)
-				shf_fprintf(shl_stdout, "%.*s\t", ++t - s, s);
+				shf_fprintf(shl_stdout, "%.*s\t",
+				    (int)(++t - s), s);
 			shf_fprintf(shl_stdout, "%s\n", s);
 		}
 		shf_flush(shl_stdout);
@@ -246,8 +230,7 @@ c_fc(char **wp)
 			return 1;
 		}
 
-		n = fstat(shf_fileno(shf), &statb) < 0 ? 128 :
-		    statb.st_size + 1;
+		n = stat(tf->name, &statb) < 0 ? 128 : statb.st_size + 1;
 		Xinit(xs, xp, n, hist_source->areap);
 		while ((n = shf_read(xp, Xnleft(xs, xp), shf)) > 0) {
 			xp += n;
@@ -480,8 +463,9 @@ findhistrel(const char *str)
 {
 	int	maxhist = histptr - history;
 	int	start = maxhist - 1;
-	int	rec = atoi(str);
+	int	rec;
 
+	getn(str, &rec);
 	if (rec == 0)
 		return -1;
 	if (rec > 0) {
@@ -517,6 +501,7 @@ sethistsize(int n)
 	}
 }
 
+#if HAVE_PERSISTENT_HISTORY
 /*
  *	set history file
  *	This can mean reloading/resetting/starting history file
@@ -550,6 +535,7 @@ sethistfile(const char *name)
 
 	hist_init(hist_source);
 }
+#endif
 
 /*
  *	initialise the history vector
@@ -578,8 +564,7 @@ init_histvec(void)
  * save command in history
  */
 void
-histsave(int lno __attribute__((unused)), const char *cmd,
-    int dowrite __attribute__((unused)))
+histsave(int lno __unused, const char *cmd, int dowrite __unused)
 {
 	char **hp;
 	char *c, *cp;
@@ -588,7 +573,7 @@ histsave(int lno __attribute__((unused)), const char *cmd,
 	if ((cp = strchr(c, '\n')) != NULL)
 		*cp = '\0';
 
-#ifdef DO_HISTORY
+#if HAVE_PERSISTENT_HISTORY
 	if (histfd && dowrite)
 		writehistfile(lno, c);
 #endif
@@ -631,7 +616,7 @@ histsave(int lno __attribute__((unused)), const char *cmd,
 void
 hist_init(Source *s)
 {
-#ifdef DO_HISTORY
+#if HAVE_PERSISTENT_HISTORY
 	unsigned char	*base;
 	int	lines;
 	int	fd;
@@ -644,7 +629,7 @@ hist_init(Source *s)
 
 	hist_source = s;
 
-#ifdef DO_HISTORY
+#if HAVE_PERSISTENT_HISTORY
 	hname = str_val(global("HISTFILE"));
 	if (hname == NULL)
 		return;
@@ -701,12 +686,10 @@ hist_init(Source *s)
 	}
 	(void) flock(histfd, LOCK_UN);
 	hsize = lseek(histfd, 0L, SEEK_END);
-#else
-	hname = NULL;
-	histfd = 0;
 #endif
 }
 
+#if HAVE_PERSISTENT_HISTORY
 typedef enum state {
 	shdr,		/* expecting a header */
 	sline,		/* looking for a null byte to end the line */
@@ -714,7 +697,6 @@ typedef enum state {
 	sn2, sn3, sn4
 } State;
 
-#ifdef DO_HISTORY
 static int
 hist_count_lines(unsigned char *base, int bytes)
 {
@@ -765,9 +747,11 @@ hist_shrink(unsigned char *oldbase, int oldbytes)
 	/*
 	 *	create temp file
 	 */
-	(void) shf_snprintf(nfile, sizeof(nfile), "%s.%d", hname, procpid);
+	shf_snprintf(nfile, sizeof(nfile), "%s.%d", hname, (int)procpid);
 	if ((fd = open(nfile, O_CREAT | O_TRUNC | O_WRONLY, 0600)) < 0)
 		return 1;
+	if (fstat(histfd, &statb) >= 0)
+		chown(nfile, statb.st_uid, statb.st_gid);
 
 	if (sprinkle(fd)) {
 		close(fd);
@@ -779,11 +763,6 @@ hist_shrink(unsigned char *oldbase, int oldbytes)
 		unlink(nfile);
 		return 1;
 	}
-	/*
-	 *	worry about who owns this file
-	 */
-	if (fstat(histfd, &statb) >= 0)
-		fchown(fd, statb.st_uid, statb.st_gid);
 	close(fd);
 
 	/*
@@ -949,56 +928,39 @@ writehistfile(int lno, char *cmd)
  bad:
 	hist_finish();
 }
-#endif
 
 void
 hist_finish(void)
 {
-#ifdef DO_HISTORY
 	(void) flock(histfd, LOCK_UN);
 	(void) close(histfd);
-#endif
 	histfd = 0;
 }
 
-#ifdef DO_HISTORY
 /*
  *	add magic to the history file
  */
 static int
 sprinkle(int fd)
 {
-	static unsigned char mag[] = { HMAGIC1, HMAGIC2 };
+	static const unsigned char mag[] = { HMAGIC1, HMAGIC2 };
 
 	return(write(fd, mag, 2) != 2);
 }
 #endif
 
-#ifdef NEED_MKSH_SIGNAME
-static const char *
-mksh_signame(int s)
-{
-	int i = 0;
-	static const struct _mksh_sigpair {
-		int nr;
-		const char *name;
-	} mksh_sigpair[] = {
-#include "signames.inc"
-		{ 0, NULL }
-	};
-
- mksh_sigscan:
-	if ((mksh_sigpair[i].nr == s) || !mksh_sigpair[i].name)
-		return (mksh_sigpair[i].name);
-	++i;
-	goto mksh_sigscan;
-}
+#if HAVE_SYS_SIGNAME
+#elif HAVE__SYS_SIGNAME
+#define sys_signame	_sys_signame
+#else
+#include "signames.c"
 #endif
 
 void
 inittraps(void)
 {
-	int	i;
+	int i;
+	const char *cs;
 
 	/* Populate sigtraps based on sys_signame and sys_siglist. */
 	for (i = 0; i <= NSIG; i++) {
@@ -1007,8 +969,36 @@ inittraps(void)
 			sigtraps[i].name = "ERR";
 			sigtraps[i].mess = "Error handler";
 		} else {
-			sigtraps[i].name = mksh_signame(i);
-			sigtraps[i].mess = mksh_siglist(i);
+#if HAVE_SYS_SIGNAME || HAVE__SYS_SIGNAME
+			cs = sys_signame[i];
+#else
+			const struct mksh_sigpair *pair = mksh_sigpairs;
+			while ((pair->nr != i) && (pair->name != NULL))
+				++pair;
+			cs = pair->name;
+#endif
+			if ((cs == NULL) ||
+			    (cs[0] == '\0'))
+				sigtraps[i].name = shf_smprintf("%d", i);
+			else {
+				char *s;
+
+				sigtraps[i].name = s = str_save(
+				    !strncasecmp(cs, "SIG", 3) ? cs + 3 : cs,
+				    APERM);
+				while ((*s = ksh_toupper(*s)))
+					++s;
+			}
+#if HAVE_SYS_SIGLIST
+			sigtraps[i].mess = sys_siglist[i];
+#elif HAVE_STRSIGNAL
+			sigtraps[i].mess = strsignal(i);
+#else
+			sigtraps[i].mess = NULL;
+#endif
+			if ((sigtraps[i].mess == NULL) ||
+			    (sigtraps[i].mess[0] == '\0'))
+				sigtraps[i].mess = shf_smprintf("Signal %d", i);
 		}
 	}
 	sigtraps[SIGEXIT_].name = "EXIT";	/* our name for signal 0 */
@@ -1044,7 +1034,7 @@ alarm_init(void)
 
 /* ARGSUSED */
 static void
-alarm_catcher(int sig __attribute__((unused)))
+alarm_catcher(int sig __unused)
 {
 	int errno_ = errno;
 
@@ -1063,32 +1053,18 @@ alarm_catcher(int sig __attribute__((unused)))
 Trap *
 gettrap(const char *name, int igncase)
 {
-	int i;
+	int n = NSIG + 1;
 	Trap *p;
 
-	if (digit(*name)) {
-		int n;
-
+	if (ksh_isdigit(*name)) {
 		if (getn(name, &n) && 0 <= n && n < NSIG)
-			return &sigtraps[n];
-		return NULL;
-	}
-	for (p = sigtraps, i = NSIG+1; --i >= 0; p++)
-		if (p->name) {
-			if (igncase) {
-				if (p->name && (!strcasecmp(p->name, name) ||
-				    (strlen(name) > 3 && !strncasecmp("SIG",
-				    p->name, 3) &&
-				    !strcasecmp(p->name, name + 3))))
-					return p;
-			} else {
-				if (p->name && (!strcmp(p->name, name) ||
-				    (strlen(name) > 3 && !strncmp("SIG",
-				    p->name, 3) && !strcmp(p->name, name + 3))))
-					return p;
-			}
-		}
-	return NULL;
+			return (&sigtraps[n]);
+	} else for (p = sigtraps; --n >= 0; p++)
+		if (!(igncase ? strcasecmp : strcmp)(p->name, name) ||
+		   (!strncasecmp(name, "SIG", 3) &&
+		    !(igncase ? strcasecmp : strcmp)(p->name, name + 3)))
+			return (p);
+	return (NULL);
 }
 
 /*
@@ -1263,7 +1239,7 @@ restoresigs(void)
 }
 
 void
-settrap(Trap *p, char *s)
+settrap(Trap *p, const char *s)
 {
 	sig_t f;
 
@@ -1383,7 +1359,7 @@ setexecsig(Trap *p, int restore)
 {
 	/* XXX debugging */
 	if (!(p->flags & (TF_ORIG_IGN|TF_ORIG_DFL)))
-		internal_errorf(1, "setexecsig: unset signal %d(%s)",
+		internal_errorf("setexecsig: unset signal %d(%s)",
 		    p->signal, p->name);
 
 	/* restore original value for exec'd kids */
