@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/jobs.c,v 1.11 2006/08/28 01:25:33 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/jobs.c,v 1.21.2.1 2007/05/13 19:29:36 tg Exp $");
 
 /* Order important! */
 #define PRUNNING	0
@@ -12,11 +12,11 @@ __RCSID("$MirOS: src/bin/mksh/jobs.c,v 1.11 2006/08/28 01:25:33 tg Exp $");
 
 typedef struct proc	Proc;
 struct proc {
-	Proc	*next;		/* next process in pipeline (if any) */
-	int	state;
-	int	status;		/* wait status */
-	pid_t	pid;		/* process id */
-	char	command[48];	/* process command string */
+	Proc *next;		/* next process in pipeline (if any) */
+	int state;
+	int status;		/* wait status */
+	pid_t pid;		/* process id */
+	char command[48];	/* process command string */
 };
 
 /* Notify/print flag - j_print() argument */
@@ -46,21 +46,21 @@ struct proc {
 
 typedef struct job Job;
 struct job {
-	Job	*next;		/* next job in list */
-	int	job;		/* job number: %n */
-	int	flags;		/* see JF_* */
-	int	state;		/* job state */
-	int	status;		/* exit status of last process */
-	pid_t	pgrp;		/* process group of job */
-	pid_t	ppid;		/* pid of process that forked job */
+	Job *next;		/* next job in list */
+	int job;		/* job number: %n */
+	int flags;		/* see JF_* */
+	int state;		/* job state */
+	int status;		/* exit status of last process */
+	pid_t pgrp;		/* process group of job */
+	pid_t ppid;		/* pid of process that forked job */
 	int32_t	age;		/* number of jobs started */
 	struct timeval systime;	/* system time used by job */
 	struct timeval usrtime;	/* user time used by job */
-	Proc	*proc_list;	/* process list */
-	Proc	*last_proc;	/* last process in list */
+	Proc *proc_list;	/* process list */
+	Proc *last_proc;	/* last process in list */
 	Coproc_id coproc_id;	/* 0 or id of coprocess output pipe */
 	struct termios ttystate;/* saved tty state for stopped jobs */
-	pid_t	saved_ttypgrp;	/* saved tty process group for stopped jobs */
+	pid_t saved_ttypgrp;	/* saved tty process group for stopped jobs */
 };
 
 /* Flags for j_waitj() */
@@ -75,7 +75,7 @@ struct job {
 #define JL_AMBIG	2	/* %foo or %?foo is ambiguous */
 #define JL_INVALID	3	/* non-pid, non-% job id */
 
-static const char	*const lookup_msgs[] = {
+static const char *const lookup_msgs[] = {
 	null,
 	"no such job",
 	"ambiguous",
@@ -83,17 +83,17 @@ static const char	*const lookup_msgs[] = {
 	NULL
 };
 
-struct timeval	j_systime, j_usrtime;	/* user and system time of last j_waitjed job */
+static Job *job_list;		/* job list */
+static Job *last_job;
+static Job *async_job;
+static pid_t async_pid;
 
-static Job		*job_list;	/* job list */
-static Job		*last_job;
-static Job		*async_job;
-static pid_t		async_pid;
-
-static int		nzombie;	/* # of zombies owned by this process */
-static int32_t		njobs;		/* # of jobs started */
-static int		child_max;	/* CHILD_MAX */
+static int nzombie;		/* # of zombies owned by this process */
+static int32_t njobs;		/* # of jobs started */
 
+#ifndef CHILD_MAX
+#define CHILD_MAX	_POSIX_CHILD_MAX
+#endif
 
 /* held_sigchld is set if sigchld occurs before a job is completely started */
 static volatile sig_atomic_t held_sigchld;
@@ -117,12 +117,10 @@ static void		put_job(Job *, int);
 static void		remove_job(Job *, const char *);
 static int		kill_job(Job *, int);
 
-/* initialize job control */
+/* initialise job control */
 void
 j_init(int mflagset)
 {
-	child_max = sysconf(_SC_CHILD_MAX);
-
 	sigemptyset(&sm_default);
 	sigprocmask(SIG_SETMASK, &sm_default, NULL);
 
@@ -321,11 +319,11 @@ exchild(struct op *t, int flags,
 	p->pid = 0;
 
 	/* link process into jobs list */
-	if (flags&XPIPEI) {	/* continuing with a pipe */
+	if (flags & XPIPEI) {	/* continuing with a pipe */
 		if (!last_job)
-			internal_errorf(1,
+			internal_errorf(
 			    "exchild: XPIPEI and no last_job - pid %d",
-			    (int) procpid);
+			    (int)procpid);
 		j = last_job;
 		last_proc->next = p;
 		last_proc = p;
@@ -429,7 +427,11 @@ exchild(struct op *t, int flags,
 		tty_close();
 		cleartraps();
 		execute(t, (flags & XERROK) | XEXEC); /* no return */
-		internal_errorf(0, "exchild: execute() returned");
+		internal_warningf("exchild: execute() returned");
+#ifndef MKSH_SMALL
+		fptreef(shl_out, 2, "exchild: tried to execute {\n%T\n}\n", t);
+		shf_flush(shl_out);
+#endif
 		unwind(LLEAVE);
 		/* NOTREACHED */
 	}
@@ -449,7 +451,8 @@ exchild(struct op *t, int flags,
 			if (Flag(FTALKING)) {
 				shf_fprintf(shl_out, "[%d]", j->job);
 				for (p = j->proc_list; p; p = p->next)
-					shf_fprintf(shl_out, " %d", p->pid);
+					shf_fprintf(shl_out, " %d",
+					    (int)p->pid);
 				shf_putchar('\n', shl_out);
 				shf_flush(shl_out);
 			}
@@ -493,7 +496,7 @@ waitlast(void)
 		if (!j)
 			warningf(true, "waitlast: no last job");
 		else
-			internal_errorf(0, "waitlast: not started");
+			internal_warningf("waitlast: not started");
 		sigprocmask(SIG_SETMASK, &omask, NULL);
 		return 125; /* not so arbitrary, non-zero value */
 	}
@@ -835,12 +838,12 @@ j_set_async(Job *j)
 	if (async_job && (async_job->flags & (JF_KNOWN|JF_ZOMBIE)) == JF_ZOMBIE)
 		remove_job(async_job, "async");
 	if (!(j->flags & JF_STARTED)) {
-		internal_errorf(0, "j_async: job not started");
+		internal_warningf("j_async: job not started");
 		return;
 	}
 	async_job = j;
 	async_pid = j->last_proc->pid;
-	while (nzombie > child_max) {
+	while (nzombie > CHILD_MAX) {
 		oldest = NULL;
 		for (jl = job_list; jl; jl = jl->next)
 			if (jl != async_job && (jl->flags & JF_ZOMBIE) &&
@@ -849,8 +852,8 @@ j_set_async(Job *j)
 		if (!oldest) {
 			/* XXX debugging */
 			if (!(async_job->flags & JF_ZOMBIE) || nzombie != 1) {
-				internal_errorf(0,
-				    "j_async: bad nzombie (%d)", nzombie);
+				internal_warningf("j_async: bad nzombie (%d)",
+				    nzombie);
 				nzombie = 0;
 			}
 			break;
@@ -1011,7 +1014,7 @@ j_waitj(Job *j,
  */
 /* ARGSUSED */
 static void
-j_sigchld(int sig __attribute__((unused)))
+j_sigchld(int sig __unused)
 {
 	int		errno_ = errno;
 	Job		*j;
@@ -1091,7 +1094,7 @@ check_job(Job *j)
 
 	/* XXX debugging (nasty - interrupt routine using shl_out) */
 	if (!(j->flags & JF_STARTED)) {
-		internal_errorf(0, "check_job: job started (flags 0x%x)",
+		internal_warningf("check_job: job started (flags 0x%x)",
 		    j->flags);
 		return;
 	}
@@ -1202,8 +1205,8 @@ j_print(Job *j, int how, struct shf *shf)
 		 * group leader (ie, !FMONITOR).  We arbitrarily return
 		 * last pid (which is what $! returns).
 		 */
-		shf_fprintf(shf, "%d\n", j->pgrp ? j->pgrp :
-		    (j->last_proc ? j->last_proc->pid : 0));
+		shf_fprintf(shf, "%d\n", (int)(j->pgrp ? j->pgrp :
+		    (j->last_proc ? j->last_proc->pid : 0)));
 		return;
 	}
 	j->flags &= ~JF_CHANGED;
@@ -1258,7 +1261,7 @@ j_print(Job *j, int how, struct shf *shf)
 		}
 
 		if (how == JP_LONG)
-			shf_fprintf(shf, "%5d ", p->pid);
+			shf_fprintf(shf, "%5d ", (int)p->pid);
 
 		if (how == JP_SHORT) {
 			if (buf[0]) {
@@ -1278,8 +1281,9 @@ j_print(Job *j, int how, struct shf *shf)
 		p = p->next;
 		while (p && p->state == state && p->status == status) {
 			if (how == JP_LONG)
-				shf_fprintf(shf, "%s%5d %-20s %s%s", filler, p->pid,
-				    space, p->command, p->next ? "|" : null);
+				shf_fprintf(shf, "%s%5d %-20s %s%s", filler,
+				    (int)p->pid, space, p->command,
+				    p->next ? "|" : null);
 			else if (how == JP_MEDIUM)
 				shf_fprintf(shf, " %s%s", p->command,
 				    p->next ? "|" : null);
@@ -1301,8 +1305,8 @@ j_lookup(const char *cp, int *ecodep)
 	Proc		*p;
 	int		len, job = 0;
 
-	if (digit(*cp)) {
-		job = atoi(cp);
+	if (ksh_isdigit(*cp)) {
+		getn(cp, &job);
 		/* Look for last_proc->pid (what $! returns) first... */
 		for (j = job_list; j != NULL; j = j->next)
 			if (j->last_proc && j->last_proc->pid == job)
@@ -1336,7 +1340,7 @@ j_lookup(const char *cp, int *ecodep)
 
 	case '0': case '1': case '2': case '3': case '4':
 	case '5': case '6': case '7': case '8': case '9':
-		job = atoi(cp);
+		getn(cp, &job);
 		for (j = job_list; j != NULL; j = j->next)
 			if (j->job == job)
 				return j;
@@ -1444,7 +1448,7 @@ remove_job(Job *j, const char *where)
 	for (; curr != NULL && curr != j; prev = &curr->next, curr = *prev)
 		;
 	if (curr != j) {
-		internal_errorf(0, "remove_job: job not found (%s)", where);
+		internal_warningf("remove_job: job not found (%s)", where);
 		return;
 	}
 	*prev = curr->next;
