@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/lex.c,v 1.21 2006/08/02 11:33:37 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/lex.c,v 1.29.2.1 2007/05/13 19:29:37 tg Exp $");
 
 /* Structure to keep track of the lexing state and the various pieces of info
  * needed for each particular state. */
@@ -42,11 +42,10 @@ struct lex_state {
 
 typedef struct State_info State_info;
 struct State_info {
-	Lex_state	*base;
-	Lex_state	*end;
+	Lex_state *base;
+	Lex_state *end;
 };
 
-
 static void	readhere(struct ioword *);
 static int	getsc__(void);
 static void	getsc_line(Source *);
@@ -63,11 +62,13 @@ static int dopprompt(const char *, int, int);
 static int backslash_skip;
 static int ignore_backslash_newline;
 
-/* optimized getsc_bn() */
+/* optimised getsc_bn() */
 #define getsc()		(*source->str != '\0' && *source->str != '\\' \
-			 && !backslash_skip ? *source->str++ : getsc_bn())
-/* optimized getsc__() */
-#define	getsc_()	((*source->str != '\0') ? *source->str++ : getsc__())
+			 && !backslash_skip && !(source->flags & SF_FIRST) \
+			 ? *source->str++ : getsc_bn())
+/* optimised getsc__() */
+#define	getsc_()	((*source->str != '\0') && !(source->flags & SF_FIRST) \
+			 ? *source->str++ : getsc__())
 
 #define STATE_BSIZE	32
 
@@ -83,10 +84,8 @@ static int ignore_backslash_newline;
 			    state = statep->ls_state; \
 			} while (0)
 
-
-
 /*
- * Lexical analyzer
+ * Lexical analyser
  *
  * tokens are not regular expressions, they are LL(1).
  * for example, "${var:-${PWD}}", and "$(size $(whence ksh))".
@@ -273,19 +272,19 @@ yylex(int cf)
 						ungetsc(c);
 						PUSH_STATE(SBRACE);
 					}
-				} else if (ctype(c, C_ALPHA)) {
+				} else if (ksh_isalphx(c)) {
 					*wp++ = OSUBST;
 					*wp++ = 'X';
 					do {
 						Xcheck(ws, wp);
 						*wp++ = c;
 						c = getsc();
-					} while (ctype(c, C_ALPHA|C_DIGIT));
+					} while (ksh_isalnux(c));
 					*wp++ = '\0';
 					*wp++ = CSUBST;
 					*wp++ = 'X';
 					ungetsc(c);
-				} else if (ctype(c, C_DIGIT|C_VAR1)) {
+				} else if (ctype(c, C_VAR1 | C_DIGIT)) {
 					Xcheck(ws, wp);
 					*wp++ = OSUBST;
 					*wp++ = 'X';
@@ -615,7 +614,7 @@ yylex(int cf)
 	dp = Xstring(ws, wp);
 	if ((c == '<' || c == '>') && state == SBASE &&
 	    ((c2 = Xlength(ws, wp)) == 0 ||
-	    (c2 == 2 && dp[0] == CHAR && digit(dp[1])))) {
+	    (c2 == 2 && dp[0] == CHAR && ksh_isdigit(dp[1])))) {
 		struct ioword *iop = (struct ioword *) alloc(sizeof(*iop), ATEMP);
 
 		if (c2 == 2)
@@ -655,41 +654,28 @@ yylex(int cf)
 	if (wp == dp && state == SBASE) {
 		Xfree(ws, wp);	/* free word */
 		/* no word, process LEX1 character */
-		switch (c) {
-		default:
-			return c;
-
-		case '|':
-		case '&':
-		case ';':
+		if ((c == '|') || (c == '&') || (c == ';') || (c == '('/*)*/)) {
 			if ((c2 = getsc()) == c)
 				c = (c == ';') ? BREAK :
 				    (c == '|') ? LOGOR :
 				    (c == '&') ? LOGAND :
+				    /*
+				     * this is the place where
+				     * ((...); (...))
+				     * and similar is broken
+				     */
+				    (c == '(' /*)*/ ) ? MDPAREN :
 				    YYERRCODE;
 			else if (c == '|' && c2 == '&')
 				c = COPROC;
 			else
 				ungetsc(c2);
-			return c;
-
-		case '\n':
+		} else if (c == '\n') {
 			gethere();
 			if (cf & CONTIN)
 				goto Again;
-			return c;
-
-		case '(':  /*)*/
-			if ((c2 = getsc()) == '(') /*)*/
-				/* XXX need to handle ((...); (...)) */
-				c = MDPAREN;
-			else
-				ungetsc(c2);
-			return c;
-		  /*(*/
-		case ')':
-			return c;
 		}
+		return (c);
 	}
 
 	*wp++ = EOS;		/* terminate word */
@@ -859,6 +845,7 @@ getsc__(void)
 	Source *s = source;
 	int c;
 
+ getsc_again:
 	while ((c = *s->str++) == 0) {
 		s->str = NULL;		/* return 0 for EOF by default */
 		switch (s->type) {
@@ -899,7 +886,7 @@ getsc__(void)
 				source->flags |= s->flags & SF_ALIAS;
 				s = source;
 			} else if (*s->u.tblp->val.s &&
-			    isspace((unsigned char)strchr(s->u.tblp->val.s, 0)[-1])) {
+			    ksh_isspace(strnul(s->u.tblp->val.s)[-1])) {
 				source = s = s->next;	/* pop source stack */
 				/* Note that this alias ended with a space,
 				 * enabling alias expansion on the following
@@ -950,6 +937,19 @@ getsc__(void)
 			shf_flush(shl_out);
 		}
 	}
+	/* check for UTF-8 byte order mark */
+	if (s->flags & SF_FIRST) {
+		s->flags &= ~SF_FIRST;
+		if (((unsigned char)c == 0xEF) &&
+		    (((const unsigned char *)(s->str))[0] == 0xBB) &&
+		    (((const unsigned char *)(s->str))[1] == 0xBF)) {
+			s->str += 2;
+#if !defined(MKSH_ASSUME_UTF8) || !defined(MKSH_SMALL)
+			Flag(FUTFHACK) = 1;
+#endif
+			goto getsc_again;
+		}
+	}
 	return c;
 }
 
@@ -1116,7 +1116,15 @@ dopprompt(const char *cp, int ntruncate, int doprint)
 				columns--;
 		} else if (*cp == delimiter)
 			indelimit = !indelimit;
-		else
+		else if (Flag(FUTFHACK) && ((unsigned)*cp > 0x7F)) {
+			const char *cp2;
+			columns += utf_widthadj(cp, &cp2);
+			if (doprint && (indelimit ||
+			    (ntruncate < (x_cols * lines + columns))))
+				shf_write(cp, cp2 - cp, shl_out);
+			cp = cp2 - /* loop increment */ 1;
+			continue;
+		} else
 			columns++;
 		if (doprint && (*cp != delimiter) &&
 		    (indelimit || (ntruncate < (x_cols * lines + columns))))
@@ -1166,9 +1174,9 @@ get_brace_var(XString *wsp, char *wp)
 			}
 			/* FALLTHRU */
 		case PS_SAW_HASH:
-			if (letter(c))
+			if (ksh_isalphx(c))
 				state = PS_IDENT;
-			else if (digit(c))
+			else if (ksh_isdigit(c))
 				state = PS_NUMBER;
 			else if (ctype(c, C_VAR1))
 				state = PS_VAR1;
@@ -1176,7 +1184,7 @@ get_brace_var(XString *wsp, char *wp)
 				state = PS_END;
 			break;
 		case PS_IDENT:
-			if (!letnum(c)) {
+			if (!ksh_isalnux(c)) {
 				state = PS_END;
 				if (c == '[') {
 					char *tmp, *p;
@@ -1194,7 +1202,7 @@ get_brace_var(XString *wsp, char *wp)
 			}
 			break;
 		case PS_NUMBER:
-			if (!digit(c))
+			if (!ksh_isdigit(c))
 				state = PS_END;
 			break;
 		case PS_VAR1:
