@@ -6,10 +6,22 @@
 #define	EXTERN
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/main.c,v 1.48 2006/08/24 20:32:53 tg Exp $");
+#if HAVE_LANGINFO_CODESET
+#include <langinfo.h>
+#endif
+#if HAVE_SETLOCALE_CTYPE
+#include <locale.h>
+#endif
+
+__RCSID("$MirOS: src/bin/mksh/main.c,v 1.74.2.1 2007/05/13 19:29:37 tg Exp $");
 
 extern char **environ;
 
+#if !HAVE_SETRESUGID
+extern uid_t kshuid;
+extern gid_t kshgid, kshegid;
+#endif
+
 static void reclaim(void);
 static void remove_temps(struct temp *);
 
@@ -48,18 +60,25 @@ static const char *initcoms[] = {
 static int initio_done;
 
 int
-main(int argc, char *argv[])
+main(int argc, const char *argv[])
 {
 	int i;
 	int argi;
 	Source *s;
 	struct block *l;
 	int restricted, errexit;
-	char **wp;
+	const char **wp;
 	struct env env;
 	pid_t ppid;
 	struct tbl *vp;
 	struct stat s_stdin;
+#if !defined(_PATH_DEFPATH) && defined(_CS_PATH)
+	size_t k;
+	char *cp;
+#endif
+#if HAVE_SETLOCALE_CTYPE
+	const char *cc;
+#endif
 
 	/* make sure argv[] is sane */
 	if (!*argv) {
@@ -67,12 +86,12 @@ main(int argc, char *argv[])
 			"mksh", NULL
 		};
 
-		argv = (char **)empty_argv;
+		argv = empty_argv;
 		argc = 1;
 	}
 	kshname = *argv;
 
-	ainit(&aperm);		/* initialize permanent Area */
+	ainit(&aperm);		/* initialise permanent Area */
 
 	/* set up base environment */
 	memset(&env, 0, sizeof(env));
@@ -99,7 +118,9 @@ main(int argc, char *argv[])
 	/* set up variable and command dictionaries */
 	ktinit(&taliases, APERM, 0);
 	ktinit(&aliases, APERM, 0);
+#ifndef MKSH_SMALL
 	ktinit(&homedirs, APERM, 0);
+#endif
 
 	/* define shell keywords */
 	initkeywords();
@@ -113,17 +134,24 @@ main(int argc, char *argv[])
 
 	init_histvec();
 
+#ifdef _PATH_DEFPATH
 	def_path = _PATH_DEFPATH;
+#else
 #ifdef _CS_PATH
-	{
-		size_t len;
-		char *new;
-
-		if ((len = confstr(_CS_PATH, NULL, 0)) > 0) {
-			confstr(_CS_PATH, new = alloc(len + 1, APERM), len + 1);
-			def_path = new;
-		}
-	}
+	if ((k = confstr(_CS_PATH, NULL, 0)) != (size_t)-1 && k > 0 &&
+	    confstr(_CS_PATH, cp = alloc(k + 1, APERM), k + 1) == k + 1)
+		def_path = cp;
+	else
+#endif
+		/*
+		 * this is uniform across all OSes unless it
+		 * breaks somewhere; don't try to optimise,
+		 * e.g. add stuff for Interix or remove /usr
+		 * for HURD, because e.g. Debian GNU/HURD is
+		 * "keeping a regular /usr"; this is supposed
+		 * to be a sane 'basic' default PATH
+		 */
+		def_path = "/bin:/usr/bin:/sbin:/usr/sbin";
 #endif
 
 	/* Set PATH to def_path (will set the path global variable).
@@ -133,7 +161,6 @@ main(int argc, char *argv[])
 	/* setstr can't fail here */
 	setstr(vp, def_path, KSH_RETURN_ERROR);
 
-
 	/* Turn on nohup by default for now - will change to off
 	 * by default once people are aware of its existence
 	 * (at&t ksh does not have a nohup option - it always sends
@@ -154,7 +181,7 @@ main(int argc, char *argv[])
 
 	/* import environment */
 	if (environ != NULL)
-		for (wp = environ; *wp != NULL; wp++)
+		for (wp = (const char **)environ; *wp != NULL; wp++)
 			typeset(*wp, IMPORT | EXPORT, 0, 0, 0);
 
 	kshpid = procpid = getpid();
@@ -188,23 +215,18 @@ main(int argc, char *argv[])
 	}
 	ppid = getppid();
 #if !HAVE_ARC4RANDOM
-	srand((*((long *)kshname)) ^ ((long)time(NULL) * kshpid * ppid));
+	srand(((long)kshname) ^ ((long)time(NULL) * kshpid * ppid));
 #endif
 	setint(global("PPID"), (long)ppid);
 
 	/* execute initialisation statements */
-	for (wp = (char **)initcoms; *wp != NULL; wp++) {
+	for (wp = initcoms; *wp != NULL; wp++) {
 		shcomexec(wp);
 		for (; *wp != NULL; wp++)
 			;
 	}
 
-	ksheuid = geteuid();
-	kshuid = getuid();
-	kshegid = getegid();
-	kshgid = getgid();
-
-	safe_prompt = ksheuid ? "$ " : "# ";
+	safe_prompt = (ksheuid = geteuid()) ? "$ " : "# ";
 	vp = global("PS1");
 	/* Set PS1 if unset or we are root and prompt doesn't contain a # */
 	if (!(vp->flag & ISSET) ||
@@ -213,7 +235,12 @@ main(int argc, char *argv[])
 		setstr(vp, safe_prompt, KSH_RETURN_ERROR);
 
 	/* Set this before parsing arguments */
-	Flag(FPRIVILEGED) = kshuid != ksheuid || kshgid != kshegid;
+#if HAVE_SETRESUGID
+	Flag(FPRIVILEGED) = getuid() != ksheuid || getgid() != getegid();
+#else
+	Flag(FPRIVILEGED) = (kshuid = getuid()) != ksheuid ||
+	    (kshgid = getgid()) != (kshegid = getegid());
+#endif
 
 	/* this to note if monitor is set on command line (see below) */
 	Flag(FMONITOR) = 127;
@@ -256,18 +283,37 @@ main(int argc, char *argv[])
 	    Flag(FTALKING))
 		reset_nonblock(0);
 
-	/* initialize job control */
+	/* initialise job control */
 	i = Flag(FMONITOR) != 127;
 	Flag(FMONITOR) = 0;
 	j_init(i);
-	/* Do this after j_init(), as tty_fd is not initialized 'til then */
-	if (Flag(FTALKING))
+	/* Do this after j_init(), as tty_fd is not initialised 'til then */
+	if (Flag(FTALKING)) {
+#ifndef MKSH_ASSUME_UTF8
+#if HAVE_SETLOCALE_CTYPE
+#define isuc(x)	(((x) != NULL) && \
+		    (stristr((x), "UTF-8") || stristr((x), "utf8")))
+		/* Check if we're in a UTF-8 locale */
+		if (!Flag(FUTFHACK)) {
+			cc = setlocale(LC_CTYPE, "");
+#if HAVE_LANGINFO_CODESET
+			if (!isuc(cc))
+				cc = nl_langinfo(CODESET);
+#endif
+			Flag(FUTFHACK) = isuc(cc);
+		}
+#undef isuc
+#endif
+#else
+		Flag(FUTFHACK) = 1;
+#endif
 		x_init();
+	}
 
 	l = e->loc;
 	l->argv = &argv[argi - 1];
 	l->argc = argc - argi;
-	l->argv[0] = (char *)kshname;
+	l->argv[0] = kshname;
 	getopts_reset(1);
 
 	/* Disable during .profile/ENV reading */
@@ -301,12 +347,12 @@ main(int argc, char *argv[])
 	}
 
 	if (restricted) {
-		static const char *const restr_com[] = {
+		static const char *restr_com[] = {
 			"typeset", "-r", "PATH",
 			"ENV", "SHELL",
 			NULL
 		};
-		shcomexec((char **)restr_com);
+		shcomexec(restr_com);
 		/* After typeset command... */
 		Flag(FRESTRICTED) = 1;
 	}
@@ -324,11 +370,11 @@ main(int argc, char *argv[])
 }
 
 int
-include(const char *name, int argc, char **argv, int intr_ok)
+include(const char *name, int argc, const char **argv, int intr_ok)
 {
 	Source *volatile s = NULL;
 	struct shf *shf;
-	char **volatile old_argv;
+	const char **volatile old_argv;
 	volatile int old_argc;
 	int i;
 
@@ -368,7 +414,7 @@ include(const char *name, int argc, char **argv, int intr_ok)
 			unwind(i);
 			/* NOTREACHED */
 		default:
-			internal_errorf(1, "include: %d", i);
+			internal_errorf("include: %d", i);
 			/* NOTREACHED */
 		}
 	}
@@ -411,6 +457,8 @@ shell(Source * volatile s, volatile int toplevel)
 	Source *volatile old_source = source;
 	int i;
 
+	s->flags |= SF_FIRST;	/* enable UTF-8 BOM check */
+
 	newenv(E_PARSE);
 	if (interactive)
 		really_exit = 0;
@@ -450,7 +498,7 @@ shell(Source * volatile s, volatile int toplevel)
 		default:
 			source = old_source;
 			quitenv(NULL);
-			internal_errorf(1, "shell: %d", i);
+			internal_errorf("shell: %d", i);
 			/* NOTREACHED */
 		}
 	}
@@ -570,8 +618,10 @@ quitenv(struct shf *shf)
 	 */
 	if (ep->oenv == NULL) {
 		if (ep->type == E_NONE) {	/* Main shell exiting? */
+#if HAVE_PERSISTENT_HISTORY
 			if (Flag(FTALKING))
 				hist_finish();
+#endif
 			j_exit();
 			if (ep->flags & EF_FAKE_SIGDIE) {
 				int sig = exstat - 128;
@@ -656,7 +706,7 @@ remove_temps(struct temp *tp)
 		}
 }
 
-/* Initialize tty_fd.  Used for saving/reseting tty modes upon
+/* Initialise tty_fd.  Used for saving/reseting tty modes upon
  * foreground job completion and for setting up tty process group.
  */
 void
@@ -731,7 +781,7 @@ errorf(const char *fmt, ...)
 
 /* like errorf(), but no unwind is done */
 void
-warningf(int fileline, const char *fmt, ...)
+warningf(bool fileline, const char *fmt, ...)
 {
 	va_list va;
 
@@ -775,25 +825,41 @@ bi_errorf(const char *fmt, ...)
 }
 
 /* Called when something that shouldn't happen does */
+static void internal_verrorf(const char *, va_list)
+    __attribute__((format (printf, 1, 0)));
+static void
+internal_verrorf(const char *fmt, va_list ap)
+{
+	shf_fprintf(shl_out, "internal error: ");
+	shf_vfprintf(shl_out, fmt, ap);
+	shf_putchar('\n', shl_out);
+	shf_flush(shl_out);
+}
+
 void
-internal_errorf(int jump, const char *fmt, ...)
+internal_errorf(const char *fmt, ...)
 {
 	va_list va;
 
-	error_prefix(true);
-	shf_fprintf(shl_out, "internal error: ");
 	va_start(va, fmt);
-	shf_vfprintf(shl_out, fmt, va);
+	internal_verrorf(fmt, va);
+	va_end(va);
+	unwind(LERROR);
+}
+
+void
+internal_warningf(const char *fmt, ...)
+{
+	va_list va;
+
+	va_start(va, fmt);
+	internal_verrorf(fmt, va);
 	va_end(va);
-	shf_putchar('\n', shl_out);
-	shf_flush(shl_out);
-	if (jump)
-		unwind(LERROR);
 }
 
 /* used by error reporting functions to print "ksh: .kshrc[25]: " */
 void
-error_prefix(int fileline)
+error_prefix(bool fileline)
 {
 	/* Avoid foo: foo[2]: ... */
 	if (!fileline || !source || !source->file ||
@@ -827,7 +893,7 @@ shprintf(const char *fmt, ...)
 	va_list va;
 
 	if (!shl_stdout_ok)
-		internal_errorf(1, "shl_stdout not valid");
+		internal_errorf("shl_stdout not valid");
 	va_start(va, fmt);
 	shf_vfprintf(shl_stdout, fmt, va);
 	va_end(va);
@@ -928,11 +994,11 @@ closepipe(int *pv)
  * a string (the X in 2>&X, read -uX, print -uX) into a file descriptor.
  */
 int
-check_fd(char *name, int mode, const char **emsgp)
+check_fd(const char *name, int mode, const char **emsgp)
 {
 	int fd, fl;
 
-	if (isdigit((unsigned char)name[0]) && !name[1]) {
+	if (ksh_isdigit(name[0]) && !name[1]) {
 		fd = name[0] - '0';
 		if ((fl = fcntl(fd = name[0] - '0', F_GETFL, 0)) < 0) {
 			if (emsgp)
@@ -1071,7 +1137,7 @@ maketemp(Area *ap, Temp_type type, struct temp **tlist)
 #define	INIT_TBLS	8	/* initial table size (power of 2) */
 
 static void texpand(struct table *, int);
-static int tnamecmp(void *, void *);
+static int tnamecmp(const void *, const void *);
 
 unsigned int
 hash(const char *n)
@@ -1214,26 +1280,28 @@ ktnext(struct tstate *ts)
 }
 
 static int
-tnamecmp(void *p1, void *p2)
+tnamecmp(const void *p1, const void *p2)
 {
-	return strcmp(((struct tbl *)p1)->name, ((struct tbl *)p2)->name);
+	const struct tbl *a = *((struct tbl * const *)p1);
+	const struct tbl *b = *((struct tbl * const *)p2);
+
+	return (strcmp(a->name, b->name));
 }
 
 struct tbl **
 ktsort(struct table *tp)
 {
-	int i;
+	size_t i;
 	struct tbl **p, **sp, **dp;
 
 	p = (struct tbl **)alloc(sizeofN(struct tbl *, tp->size + 1), ATEMP);
 	sp = tp->tbls;		/* source */
 	dp = p;			/* dest */
-	for (i = 0; i < tp->size; i++)
+	for (i = 0; i < (size_t)tp->size; i++)
 		if ((*dp = *sp++) != NULL && (((*dp)->flag & DEFINED) ||
 		    ((*dp)->flag & ARRAY)))
 			dp++;
-	i = dp - p;
-	qsortp((void **)p, (size_t)i, tnamecmp);
+	qsort(p, (i = dp - p), sizeof (void *), tnamecmp);
 	p[i] = NULL;
 	return p;
 }
