@@ -2,16 +2,31 @@
 /*	$OpenBSD: path.c,v 1.12 2005/03/30 17:16:37 deraadt Exp $	*/
 
 #include "sh.h"
+#if HAVE_GRP_H
+#include <grp.h>
+#endif
 
-__RCSID("$MirOS: src/bin/mksh/misc.c,v 1.16 2006/08/24 20:32:53 tg Exp $\t"
+__RCSID("$MirOS: src/bin/mksh/misc.c,v 1.52.2.1 2007/05/13 19:29:38 tg Exp $\t"
 	MKSH_SH_H_ID);
 
-short chtypes[UCHAR_MAX+1];	/* type bits for unsigned char */
+#undef USE_CHVT
+#if defined(TIOCSCTTY) && !defined(MKSH_SMALL)
+#define USE_CHVT
+#endif
+
+unsigned char chtypes[UCHAR_MAX + 1];	/* type bits for unsigned char */
+
+#if !HAVE_SETRESUGID
+uid_t kshuid;
+gid_t kshgid, kshegid;
+#endif
 
 static int do_gmatch(const unsigned char *, const unsigned char *,
     const unsigned char *, const unsigned char *);
 static const unsigned char *cclass(const unsigned char *, int);
-static int parse_T(char *);
+#ifdef USE_CHVT
+static void chvt(const char *);
+#endif
 static char *do_phys_path(XString *, char *, const char *);
 
 /*
@@ -46,41 +61,9 @@ initctypes(void)
 	setctypes("*@#!$-?", C_VAR1);
 	setctypes(" \t\n", C_IFSWS);
 	setctypes("=-+?", C_SUBOP1);
-	setctypes("#%", C_SUBOP2);
 	setctypes(" \n\t\"#$&'()*;<>?[]\\`|", C_QUOTE);
 }
 
-/* convert unsigned long to base N string */
-
-char *
-ulton(long unsigned int n, int base)
-{
-	char *p;
-	static char buf [20];
-
-	p = &buf[sizeof(buf)];
-	*--p = '\0';
-	do {
-		*--p = "0123456789ABCDEF"[n%base];
-		n /= base;
-	} while (n != 0);
-	return p;
-}
-
-char *
-str_save(const char *s, Area *ap)
-{
-	size_t len;
-	char *p;
-
-	if (!s)
-		return NULL;
-	len = strlen(s)+1;
-	p = alloc(len, ap);
-	strlcpy(p, s, len+1);
-	return (p);
-}
-
 /* Allocate a string of size n+1 and copy upto n characters from the possibly
  * null terminated string s into it.  Always returns a null terminated string
  * (unless n < 0).
@@ -88,20 +71,26 @@ str_save(const char *s, Area *ap)
 char *
 str_nsave(const char *s, int n, Area *ap)
 {
-	char *ns;
+	char *ns = NULL;
 
-	if (n < 0)
-		return 0;
-	ns = alloc(n + 1, ap);
-	ns[0] = '\0';
-	return strncat(ns, s, n);
+	if (n >= 0 && s)
+		strlcpy(ns = alloc(n + 1, ap), s, n + 1);
+	return (ns);
 }
 
+#ifdef MKSH_SMALL
+char *
+str_save(const char *s, Area *ap)
+{
+	return (str_nsave(s, s ? strlen(s) : 0, ap));
+}
+#endif
+
 /* called from XcheckN() to grow buffer */
 char *
-Xcheck_grow_(XString *xsp, char *xp, unsigned more)
+Xcheck_grow_(XString *xsp, const char *xp, unsigned more)
 {
-	char *old_beg = xsp->beg;
+	const char *old_beg = xsp->beg;
 
 	xsp->len += more > xsp->len ? more : xsp->len;
 	xsp->beg = aresize(xsp->beg, xsp->len + 8, xsp->areap);
@@ -119,7 +108,6 @@ const struct option options[] = {
 	{ "bgnice",	  0,		OF_ANY },
 	{ NULL,		'c',	    OF_CMDLINE },
 	{ "emacs",	  0,		OF_ANY },
-	{ "emacs-usemeta",  0,		OF_ANY }, /* non-standard */
 	{ "errexit",	'e',		OF_ANY },
 	{ "gmacs",	  0,		OF_ANY },
 	{ "ignoreeof",	  0,		OF_ANY },
@@ -139,13 +127,12 @@ const struct option options[] = {
 	{ "posix",	  0,		OF_ANY }, /* non-standard */
 	{ "privileged",	'p',		OF_ANY },
 	{ "restricted",	'r',	    OF_CMDLINE },
-	{ "sh",		  0,		OF_ANY }, /* non-standard */
 	{ "stdin",	's',	    OF_CMDLINE }, /* pseudo non-standard */
 	{ "trackall",	'h',		OF_ANY },
+	{ "utf8-hack",	'U',		OF_ANY }, /* non-standard */
 	{ "verbose",	'v',		OF_ANY },
 	{ "vi",		  0,		OF_ANY },
 	{ "viraw",	  0,		OF_ANY }, /* no effect */
-	{ "vi-show8",	  0,		OF_ANY }, /* non-standard */
 	{ "vi-tabcomplete",  0,		OF_ANY }, /* non-standard */
 	{ "vi-esccomplete",  0,		OF_ANY }, /* non-standard */
 	{ "xtrace",	'x',		OF_ANY },
@@ -158,38 +145,35 @@ const struct option options[] = {
 /*
  * translate -o option into F* constant (also used for test -o option)
  */
-int
+size_t
 option(const char *n)
 {
-	unsigned i;
+	size_t i;
 
 	for (i = 0; i < NELEM(options); i++)
 		if (options[i].name && strcmp(options[i].name, n) == 0)
-			return i;
+			return (i);
 
-	return -1;
+	return ((size_t)-1);
 }
 
 struct options_info {
 	int opt_width;
-	struct {
-		const char *name;
-		int	flag;
-	} opts[NELEM(options)];
+	int opts[NELEM(options)];
 };
 
-static char *options_fmt_entry(void *arg, int i, char *buf, int buflen);
-static void printoptions(int verbose);
+static char *options_fmt_entry(const void *arg, int, char *, int);
+static void printoptions(int);
 
 /* format a single select menu item */
 static char *
-options_fmt_entry(void *arg, int i, char *buf, int buflen)
+options_fmt_entry(const void *arg, int i, char *buf, int buflen)
 {
-	struct options_info *oi = (struct options_info *) arg;
+	const struct options_info *oi = (const struct options_info *)arg;
 
 	shf_snprintf(buf, buflen, "%-*s %s",
-	    oi->opt_width, oi->opts[i].name,
-	    Flag(oi->opts[i].flag) ? "on" : "off");
+	    oi->opt_width, options[oi->opts[i]].name,
+	    Flag(oi->opts[i]) ? "on" : "off");
 	return buf;
 }
 
@@ -208,8 +192,7 @@ printoptions(int verbose)
 		for (i = n = oi.opt_width = 0; i < NELEM(options); i++)
 			if (options[i].name) {
 				len = strlen(options[i].name);
-				oi.opts[n].name = options[i].name;
-				oi.opts[n++].flag = i;
+				oi.opts[n++] = i;
 				if (len > oi.opt_width)
 					oi.opt_width = len;
 			}
@@ -247,10 +230,6 @@ change_flag(enum sh_flag f,
 {
 	int oldval;
 
-	/* limited pdksh compatibility (FSH stays always off) */
-	if (f == FSH)
-		return;
-
 	oldval = Flag(f);
 	Flag(f) = newval;
 	if (f == FMONITOR) {
@@ -261,10 +240,20 @@ change_flag(enum sh_flag f,
 		Flag(f) = newval;
 	} else if (f == FPRIVILEGED && oldval && !newval) {
 		/* Turning off -p? */
+#if HAVE_SETRESUGID
+		gid_t kshegid = getgid();
+
+		setresgid(kshegid, kshegid, kshegid);
+#if HAVE_SETGROUPS
+		setgroups(1, &kshegid);
+#endif
+		setresuid(ksheuid, ksheuid, ksheuid);
+#else
 		seteuid(ksheuid = kshuid = getuid());
 		setuid(ksheuid);
 		setegid(kshegid = kshgid = getgid());
 		setgid(kshegid);
+#endif
 	} else if (f == FPOSIX && newval) {
 		Flag(FBRACEEXPAND) = 0;
 	}
@@ -279,28 +268,36 @@ change_flag(enum sh_flag f,
  * non-option arguments, -1 if there is an error.
  */
 int
-parse_args(char **argv,
+parse_args(const char **argv,
     int what,			/* OF_CMDLINE or OF_SET */
     int *setargsp)
 {
 	static char cmd_opts[NELEM(options) + 5]; /* o:T:\0 */
-	static char set_opts[NELEM(options) + 5]; /* Ao;s\0 */
+	static char set_opts[NELEM(options) + 6]; /* A:o;s\0 */
 	char *opts;
-	char *array = NULL;
+	const char *array = NULL;
 	Getopt go;
+	size_t i;
 	int optc, set, sortargs = 0, arrayset = 0;
-	unsigned i;
 
 	/* First call?  Build option strings... */
 	if (cmd_opts[0] == '\0') {
-		char *p, *q;
+		char *p = cmd_opts, *q = set_opts;
 
 		/* see cmd_opts[] declaration */
-		strlcpy(cmd_opts, "o:T:", sizeof cmd_opts);
-		p = cmd_opts + strlen(cmd_opts);
+		*p++ = 'o';
+		*p++ = ':';
+#ifndef MKSH_SMALL
+		*p++ = 'T';
+		*p++ = ':';
+#endif
 		/* see set_opts[] declaration */
-		strlcpy(set_opts, "A:o;s", sizeof set_opts);
-		q = set_opts + strlen(set_opts);
+		*q++ = 'A';
+		*q++ = ':';
+		*q++ = 'o';
+		*q++ = ';';
+		*q++ = 's';
+
 		for (i = 0; i < NELEM(options); i++) {
 			if (options[i].c) {
 				if (options[i].flags & OF_CMDLINE)
@@ -314,12 +311,15 @@ parse_args(char **argv,
 	}
 
 	if (what == OF_CMDLINE) {
-		char *p;
+		const char *p = argv[0], *q;
 		/* Set FLOGIN before parsing options so user can clear
 		 * flag using +l.
 		 */
-		Flag(FLOGIN) = (argv[0][0] == '-' ||
-		    ((p = strrchr(argv[0], '/')) && *++p == '-'));
+		if (*p != '-')
+			for (q = p; *q; )
+				if (*q++ == '/')
+					p = q;
+		Flag(FLOGIN) = (*p == '-');
 		opts = cmd_opts;
 	} else if (what == OF_FIRSTTIME) {
 		opts = cmd_opts;
@@ -350,28 +350,33 @@ parse_args(char **argv,
 				break;
 			}
 			i = option(go.optarg);
-			if (i != (unsigned)-1 && set == Flag(i))
+			if ((i != (size_t)-1) && set == Flag(i))
 				/* Don't check the context if the flag
 				 * isn't changing - makes "set -o interactive"
 				 * work if you're already interactive.  Needed
 				 * if the output of "set +o" is to be used.
 				 */
 				;
-			else if (i != (unsigned)-1 && (options[i].flags & what))
-				change_flag((enum sh_flag) i, what, set);
+			else if ((i != (size_t)-1) && (options[i].flags & what))
+				change_flag((enum sh_flag)i, what, set);
 			else {
 				bi_errorf("%s: bad option", go.optarg);
-				return -1;
+				return (-1);
 			}
 			break;
 
+#ifndef MKSH_SMALL
 		case 'T':
 			if (what != OF_FIRSTTIME)
 				break;
-			if (parse_T(go.optarg))
-				return -1;
+#ifndef USE_CHVT
+			errorf("no TIOCSCTTY ioctl");
+#else
 			change_flag(FTALKING, OF_CMDLINE, 1);
+			chvt(go.optarg);
+#endif
 			break;
+#endif
 
 		case '?':
 			return -1;
@@ -391,10 +396,8 @@ parse_args(char **argv,
 					    set);
 					break;
 				}
-			if (i == NELEM(options)) {
-				internal_errorf(1, "parse_args: '%c'", optc);
-				return -1; /* not reached */
-			}
+			if (i == NELEM(options))
+				internal_errorf("parse_args: '%c'", optc);
 		}
 	}
 	if (!(go.info & GI_MINUSMINUS) && argv[go.optind] &&
@@ -418,7 +421,7 @@ parse_args(char **argv,
 	if (sortargs) {
 		for (i = go.optind; argv[i]; i++)
 			;
-		qsortp((void **) &argv[go.optind], (size_t) (i - go.optind),
+		qsort(&argv[go.optind], i - go.optind, sizeof (void *),
 		    xstrcmp);
 	}
 	if (arrayset) {
@@ -432,18 +435,36 @@ parse_args(char **argv,
 
 /* parse a decimal number: returns 0 if string isn't a number, 1 otherwise */
 int
-getn(const char *as, int *ai)
+getn(const char *s, int *ai)
 {
-	char *p;
-	long n;
+	int i, c, rv = 0;
+	bool neg = false;
 
-	n = strtol(as, &p, 10);
+	do {
+		c = *s++;
+	} while (ksh_isspace(c));
+	if (c == '-') {
+		neg = true;
+		c = *s++;
+	} else if (c == '+')
+		c = *s++;
+	*ai = i = 0;
+	do {
+		if (!ksh_isdigit(c))
+			goto getn_out;
+		i *= 10;
+		if (i < *ai)
+			/* overflow */
+			goto getn_out;
+		i += c - '0';
+		*ai = i;
+	} while ((c = *s++));
+	rv = 1;
 
-	if (!*as || *p || INT_MIN >= n || n >= INT_MAX)
-		return 0;
-
-	*ai = (int)n;
-	return 1;
+ getn_out:
+	if (neg)
+		*ai = -*ai;
+	return (rv);
 }
 
 /* getn() that prints error */
@@ -538,7 +559,7 @@ has_globbing(const char *xp, const char *xpe)
 					return 0;
 				in_bracket = 0;
 			}
-		} else if ((c & 0x80) && strchr("*+?@! ", c & 0x7f)) {
+		} else if ((c & 0x80) && vstrchr("*+?@! ", c & 0x7f)) {
 			saw_glob = 1;
 			if (in_bracket)
 				bnest++;
@@ -603,8 +624,7 @@ do_gmatch(const unsigned char *s, const unsigned char *se,
 
 		  /*
 		   * [*+?@!](pattern|pattern|..)
-		   *
-		   * Not ifdef'd KSH as this is needed for ${..%..}, etc.
+		   * This is also needed for ${..%..}, etc.
 		   */
 		case 0x80|'+': /* matches one or more times */
 		case 0x80|'*': /* matches zero or more times */
@@ -739,111 +759,19 @@ pat_scan(const unsigned char *p, const unsigned char *pe, int match_sep)
 		if ((*++p == /*(*/ ')' && nest-- == 0) ||
 		    (*p == '|' && match_sep && nest == 0))
 			return ++p;
-		if ((*p & 0x80) && strchr("*+?@! ", *p & 0x7f))
+		if ((*p & 0x80) && vstrchr("*+?@! ", *p & 0x7f))
 			nest++;
 	}
 	return NULL;
 }
 
-
-/* -------- qsort.c -------- */
-
-/*
- * quick sort of array of generic pointers to objects.
- */
-static void qsort1(void **base, void **lim, int (*f)(void *, void *));
-
-void
-qsortp(void **base,			/* base address */
-    size_t n,				/* elements */
-    int (*f) (void *, void *))		/* compare function */
-{
-	qsort1(base, base + n, f);
-}
-
-#define	swap2(a, b)	{\
-	void *t; t = *(a); *(a) = *(b); *(b) = t;\
-}
-#define	swap3(a, b, c)	{\
-	void *t; t = *(a); *(a) = *(c); *(c) = *(b); *(b) = t;\
-}
-
-static void
-qsort1(void **base, void **lim, int (*f) (void *, void *))
-{
-	void **i, **j;
-	void **lptr, **hptr;
-	size_t n;
-	int c;
-
- top:
-	n = (lim - base) / 2;
-	if (n == 0)
-		return;
-	hptr = lptr = base+n;
-	i = base;
-	j = lim - 1;
-
-	for (;;) {
-		if (i < lptr) {
-			if ((c = (*f)(*i, *lptr)) == 0) {
-				lptr --;
-				swap2(i, lptr);
-				continue;
-			}
-			if (c < 0) {
-				i += 1;
-				continue;
-			}
-		}
-
- begin:
-		if (j > hptr) {
-			if ((c = (*f)(*hptr, *j)) == 0) {
-				hptr ++;
-				swap2(hptr, j);
-				goto begin;
-			}
-			if (c > 0) {
-				if (i == lptr) {
-					hptr ++;
-					swap3(i, hptr, j);
-					i = lptr += 1;
-					goto begin;
-				}
-				swap2(i, j);
-				j -= 1;
-				i += 1;
-				continue;
-			}
-			j -= 1;
-			goto begin;
-		}
-
-		if (i == lptr) {
-			if (lptr-base >= lim-hptr) {
-				qsort1(hptr+1, lim, f);
-				lim = lptr;
-			} else {
-				qsort1(base, lptr, f);
-				base = hptr+1;
-			}
-			goto top;
-		}
-
-		lptr -= 1;
-		swap3(j, lptr, i);
-		j = hptr -= 1;
-	}
-}
-
 int
-xstrcmp(void *p1, void *p2)
+xstrcmp(const void *p1, const void *p2)
 {
-	return (strcmp((char *)p1, (char *)p2));
+	return (strcmp(*(char * const *)p1, *(char * const *)p2));
 }
 
-/* Initialize a Getopt structure */
+/* Initialise a Getopt structure */
 void
 ksh_getopt_reset(Getopt *go, int flags)
 {
@@ -881,13 +809,13 @@ ksh_getopt_reset(Getopt *go, int flags)
  *	  in go->info.
  */
 int
-ksh_getopt(char **argv, Getopt *go, const char *optionsp)
+ksh_getopt(const char **argv, Getopt *go, const char *optionsp)
 {
 	char c;
-	char *o;
+	const char *o;
 
 	if (go->p == 0 || (c = argv[go->optind - 1][go->p]) == '\0') {
-		char *arg = argv[go->optind], flag = arg ? *arg : '\0';
+		const char *arg = argv[go->optind], flag = arg ? *arg : '\0';
 
 		go->p = 1;
 		if (flag == '-' && arg[1] == '-' && arg[2] == '\0') {
@@ -909,7 +837,7 @@ ksh_getopt(char **argv, Getopt *go, const char *optionsp)
 	}
 	go->p++;
 	if (c == '?' || c == ':' || c == ';' || c == ',' || c == '#' ||
-	    !(o = strchr(optionsp, c))) {
+	    !(o = cstrchr(optionsp, c))) {
 		if (optionsp[0] == ':') {
 			go->buf[0] = c;
 			go->optarg = go->buf;
@@ -958,13 +886,13 @@ ksh_getopt(char **argv, Getopt *go, const char *optionsp)
 		 * argument is missing.
 		 */
 		if (argv[go->optind - 1][go->p]) {
-			if (digit(argv[go->optind - 1][go->p])) {
+			if (ksh_isdigit(argv[go->optind - 1][go->p])) {
 				go->optarg = argv[go->optind - 1] + go->p;
 				go->p = 0;
 			} else
 				go->optarg = NULL;
 		} else {
-			if (argv[go->optind] && digit(argv[go->optind][0])) {
+			if (argv[go->optind] && ksh_isdigit(argv[go->optind][0])) {
 				go->optarg = argv[go->optind++];
 				go->p = 0;
 			} else
@@ -994,26 +922,27 @@ print_value_quoted(const char *s)
 	}
 	for (p = s; *p; p++) {
 		if (*p == '\'') {
-			shprintf("'\\'" + 1 - inquote);
+			if (inquote)
+				shf_putc('\'', shl_stdout);
+			shf_putc('\\', shl_stdout);
 			inquote = 0;
-		} else {
-			if (!inquote) {
-				shprintf("'");
-				inquote = 1;
-			}
-			shf_putc(*p, shl_stdout);
+		} else if (!inquote) {
+			shf_putc('\'', shl_stdout);
+			inquote = 1;
 		}
+		shf_putc(*p, shl_stdout);
 	}
 	if (inquote)
-		shprintf("'");
+		shf_putc('\'', shl_stdout);
 }
 
 /* Print things in columns and rows - func() is called to format the ith
  * element
  */
 void
-print_columns(struct shf *shf, int n, char *(*func) (void *, int, char *, int),
-    void *arg, int max_width, int prefcol)
+print_columns(struct shf *shf, int n,
+    char *(*func) (const void *, int, char *, int),
+    const void *arg, int max_width, int prefcol)
 {
 	char *str = (char *) alloc(max_width + 1, ATEMP);
 	int i;
@@ -1058,7 +987,7 @@ print_columns(struct shf *shf, int n, char *(*func) (void *, int, char *, int),
 }
 
 /* Strip any nul bytes from buf - returns new length (nbytes - # of nuls) */
-int
+void
 strip_nuls(char *buf, int nbytes)
 {
 	char *dst;
@@ -1081,9 +1010,7 @@ strip_nuls(char *buf, int nbytes)
 			dst += q - p;
 		}
 		*dst = '\0';
-		return dst - buf;
 	}
-	return nbytes;
 }
 
 /* Like read(2), but if read fails due to non-blocking flag, resets flag
@@ -1097,12 +1024,11 @@ blocking_read(int fd, char *buf, int nbytes)
 
 	while ((ret = read(fd, buf, nbytes)) < 0) {
 		if (!tried_reset && errno == EAGAIN) {
-			int oerrno = errno;
 			if (reset_nonblock(fd) > 0) {
 				tried_reset = 1;
 				continue;
 			}
-			errno = oerrno;
+			errno = EAGAIN;
 		}
 		break;
 	}
@@ -1131,29 +1057,18 @@ reset_nonblock(int fd)
 
 /* Like getcwd(), except bsize is ignored if buf is 0 (PATH_MAX is used) */
 char *
-ksh_get_wd(char *buf, int bsize)
+ksh_get_wd(size_t *dlen)
 {
-	char *b;
-	char *ret;
+	char *ret, *b;
+	size_t len = 1;
 
-	/* Note: we could just use plain getcwd(), but then we'd had to
-	 * inject possibly allocated space into the ATEMP area. */
-	/* Assume getcwd() available */
-	if (!buf) {
-		bsize = PATH_MAX;
-		b = alloc(PATH_MAX + 1, ATEMP);
-	} else
-		b = buf;
-
-	ret = getcwd(b, bsize);
-
-	if (!buf) {
-		if (ret)
-			ret = aresize(b, strlen(b) + 1, ATEMP);
-		else
-			afree(b, ATEMP);
-	}
+	if ((ret = getcwd((b = alloc(PATH_MAX + 1, ATEMP)), PATH_MAX)))
+		ret = aresize(b, len = (strlen(b) + 1), ATEMP);
+	else
+		afree(b, ATEMP);
 
+	if (dlen)
+		*dlen = len;
 	return ret;
 }
 
@@ -1317,13 +1232,14 @@ simplify_path(char *pathl)
 void
 set_current_wd(char *pathl)
 {
-	int len;
+	size_t len = 1;
 	char *p = pathl;
 
-	if (!p && !(p = ksh_get_wd(NULL, 0)))
-		p = null;
-
-	len = strlen(p) + 1;
+	if (p == NULL) {
+		if ((p = ksh_get_wd(&len)) == NULL)
+			p = null;
+	} else
+		len = strlen(p) + 1;
 
 	if (len > current_wd_size)
 		current_wd = aresize(current_wd, current_wd_size = len, APERM);
@@ -1366,7 +1282,7 @@ do_phys_path(XString *xsp, char *xp, const char *pathl)
 			p++;
 		if (!*p)
 			break;
-		len = (q = strchr(p, '/')) ? q - p : (int)strlen(p);
+		len = (q = cstrchr(p, '/')) ? q - p : (int)strlen(p);
 		if (len == 1 && p[0] == '.')
 			continue;
 		if (len == 2 && p[0] == '.' && p[1] == '.') {
@@ -1403,80 +1319,109 @@ do_phys_path(XString *xsp, char *xp, const char *pathl)
 	return xp;
 }
 
-#if !defined(TIOCSCTTY)
-#define NO_CHVT "no TIOCSCTTY ioctl"
-#else
-static const char *
-chvt(char *f)
+#ifdef USE_CHVT
+static void
+chvt(const char *fn)
 {
+	char dv[20];
+	struct stat sb;
 	int fd;
 
-	if (chown(f, 0, 0))
-		return "chown";
-	if (chmod(f, 0600))
-		return "chmod";
-#if !defined(__sun__) && !defined(__gnu_linux__) && !defined(__INTERIX)
-	if (revoke(f))
-		return "revoke";
+	if (stat(fn, &sb)) {
+		memcpy(dv, "/dev/ttyC", 9);
+		strlcpy(dv + 9, fn, 20 - 9);
+		if (stat(dv, &sb)) {
+			strlcpy(dv + 8, fn, 20 - 8);
+			if (stat(dv, &sb))
+				errorf("chvt: can't find tty %s", fn);
+		}
+		fn = dv;
+	}
+	if (!(sb.st_mode & S_IFCHR))
+		errorf("chvt: not a char device: %s", fn);
+	if ((sb.st_uid != 0) && chown(fn, 0, 0))
+		warningf(false, "chvt: cannot chown root %s", fn);
+	if (((sb.st_mode & 07777) != 0600) && chmod(fn, 0600))
+		warningf(false, "chvt: cannot chmod 0600 %s", fn);
+#if HAVE_REVOKE
+	if (revoke(fn))
 #endif
+		warningf(false, "chvt: cannot revoke %s, new shell is"
+		    " potentially insecure", fn);
 
-	if ((fd = open(f, O_RDWR)) == -1) {
+	if ((fd = open(fn, O_RDWR)) == -1) {
 		sleep(1);
-		if ((fd = open(f, O_RDWR)) == -1)
-			return "open";
+		if ((fd = open(fn, O_RDWR)) == -1)
+			errorf("chvt: cannot open %s", fn);
 	}
 	switch (fork()) {
 	case -1:
-		return "fork";
+		errorf("fork failed");
 	case 0:
 		break;
 	default:
-		_exit(0);
+		exit(0);
 	}
 	if (setsid() == -1)
-		return "setsid";
+		errorf("chvt: setsid failed");
 	if (ioctl(fd, TIOCSCTTY, NULL) == -1)
-		return "ioctl";
+		errorf("chvt: TIOCSCTTY failed");
 	dup2(fd, 0);
 	dup2(fd, 1);
 	dup2(fd, 2);
 	if (fd > 2)
 		close(fd);
-
-	return NULL;
 }
 #endif
 
-static int
-parse_T(char *fn __attribute__((unused)))
+#ifdef DEBUG
+char *
+strchr(char *p, int ch)
 {
-#ifdef NO_CHVT
-	warningf(0, "chvt: %s", NO_CHVT);
-	return -1;
-#else
-	const char *rv;
-	char dv[20];
-	struct stat sb;
-
-	strlcpy(dv, fn, 20);
-	if (stat(dv, &sb)) {
-		snprintf(dv, 20, "/dev/ttyC%s", fn);
-		if (stat(dv, &sb)) {
-			snprintf(dv, 20, "/dev/tty%s", fn);
-			if (stat(dv, &sb)) {
-				warningf(0, "chvt: can't find tty %s", fn);
-				return -1;
-			}
-		}
+	for (;; ++p) {
+		if (*p == ch)
+			return (p);
+		if (!*p)
+			return (NULL);
 	}
-	if (!(sb.st_mode & S_IFCHR)) {
-		warningf(0, "chvt: not a char device: %s", fn);
-		return -1;
-	}
-	if ((rv = chvt(dv)) != NULL) {
-		warningf(0, "chvt: failed to %s: %s", rv, strerror(errno));
-		return -1;
-	}
-	return 0;
-#endif
+	/* NOTREACHED */
 }
+
+char *
+strstr(char *b, const char *l)
+{
+	char first, c;
+	size_t n;
+
+	if ((first = *l++) == '\0')
+		return (b);
+	n = strlen(l);
+ strstr_look:
+	while ((c = *b++) != first)
+		if (c == '\0')
+			return (NULL);
+	if (strncmp(b, l, n))
+		goto strstr_look;
+	return (b - 1);
+}
+#endif
+
+#if !HAVE_STRCASESTR
+const char *
+stristr(const char *b, const char *l)
+{
+	char first, c;
+	size_t n;
+
+	if ((first = *l++), ((first = ksh_tolower(first)) == '\0'))
+		return (b);
+	n = strlen(l);
+ stristr_look:
+	while ((c = *b++), ((c = ksh_tolower(c)) != first))
+		if (c == '\0')
+			return (NULL);
+	if (strncasecmp(b, l, n))
+		goto stristr_look;
+	return (b - 1);
+}
+#endif
