@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/shf.c,v 1.6 2006/08/02 12:50:20 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/shf.c,v 1.13.2.1 2007/05/13 19:29:39 tg Exp $");
 
 /* flags to shf_emptybuf() */
 #define EB_READSW	0x01	/* about to switch to reading */
@@ -14,8 +14,8 @@ __RCSID("$MirOS: src/bin/mksh/shf.c,v 1.6 2006/08/02 12:50:20 tg Exp $");
  * file descriptors.
  */
 
-static int	shf_fillbuf(struct shf *);
-static int	shf_emptybuf(struct shf *, int);
+static int shf_fillbuf(struct shf *);
+static int shf_emptybuf(struct shf *, int);
 
 /* Open a file.  First three args are for open(), last arg is flags for
  * this package.  Returns NULL if file could not be opened, or if a dup
@@ -88,7 +88,7 @@ shf_fdopen(int fd, int sflags, struct shf *shf)
 	}
 
 	if (!(sflags & (SHF_RD | SHF_WR)))
-		internal_errorf(1, "shf_fdopen: missing read/write");
+		internal_errorf("shf_fdopen: missing read/write");
 
 	if (shf) {
 		if (bsize) {
@@ -145,9 +145,9 @@ shf_reopen(int fd, int sflags, struct shf *shf)
 	}
 
 	if (!(sflags & (SHF_RD | SHF_WR)))
-		internal_errorf(1, "shf_reopen: missing read/write");
+		internal_errorf("shf_reopen: missing read/write");
 	if (!shf || !shf->buf || shf->bsize < bsize)
-		internal_errorf(1, "shf_reopen: bad shf/buf/bsize");
+		internal_errorf("shf_reopen: bad shf/buf/bsize");
 
 	/* assumes shf->buf and shf->bsize already set up */
 	shf->fd = fd;
@@ -177,7 +177,7 @@ shf_sopen(char *buf, int bsize, int sflags, struct shf *shf)
 	/* can't have a read+write string */
 	if (!(sflags & (SHF_RD | SHF_WR)) ||
 	    (sflags & (SHF_RD | SHF_WR)) == (SHF_RD | SHF_WR))
-		internal_errorf(1, "shf_sopen: flags 0x%x", sflags);
+		internal_errorf("shf_sopen: flags 0x%x", sflags);
 
 	if (!shf) {
 		shf = (struct shf *) alloc(sizeof(struct shf), ATEMP);
@@ -270,7 +270,7 @@ shf_flush(struct shf *shf)
 		return (shf->flags & SHF_WR) ? EOF : 0;
 
 	if (shf->fd < 0)
-		internal_errorf(1, "shf_flush: no fd");
+		internal_errorf("shf_flush: no fd");
 
 	if (shf->flags & SHF_ERROR) {
 		errno = shf->errno_;
@@ -300,7 +300,7 @@ shf_emptybuf(struct shf *shf, int flags)
 	int ret = 0;
 
 	if (!(shf->flags & SHF_STRING) && shf->fd < 0)
-		internal_errorf(1, "shf_emptybuf: no fd");
+		internal_errorf("shf_emptybuf: no fd");
 
 	if (shf->flags & SHF_ERROR) {
 		errno = shf->errno_;
@@ -381,7 +381,7 @@ shf_fillbuf(struct shf *shf)
 		return 0;
 
 	if (shf->fd < 0)
-		internal_errorf(1, "shf_fillbuf: no fd");
+		internal_errorf("shf_fillbuf: no fd");
 
 	if (shf->flags & (SHF_EOF | SHF_ERROR)) {
 		if (shf->flags & SHF_ERROR)
@@ -427,10 +427,10 @@ shf_read(char *buf, int bsize, struct shf *shf)
 	int ncopy;
 
 	if (!(shf->flags & SHF_RD))
-		internal_errorf(1, "shf_read: flags %x", shf->flags);
+		internal_errorf("shf_read: flags %x", shf->flags);
 
 	if (bsize <= 0)
-		internal_errorf(1, "shf_read: bsize %d", bsize);
+		internal_errorf("shf_read: bsize %d", bsize);
 
 	while (bsize > 0) {
 		if (shf->rnleft == 0 &&
@@ -462,7 +462,7 @@ shf_getse(char *buf, int bsize, struct shf *shf)
 	char *orig_buf = buf;
 
 	if (!(shf->flags & SHF_RD))
-		internal_errorf(1, "shf_getse: flags %x", shf->flags);
+		internal_errorf("shf_getse: flags %x", shf->flags);
 
 	if (bsize <= 0)
 		return NULL;
@@ -497,7 +497,7 @@ int
 shf_getchar(struct shf *shf)
 {
 	if (!(shf->flags & SHF_RD))
-		internal_errorf(1, "shf_getchar: flags %x", shf->flags);
+		internal_errorf("shf_getchar: flags %x", shf->flags);
 
 	if (shf->rnleft == 0 && (shf_fillbuf(shf) == EOF || shf->rnleft == 0))
 		return EOF;
@@ -512,7 +512,7 @@ int
 shf_ungetc(int c, struct shf *shf)
 {
 	if (!(shf->flags & SHF_RD))
-		internal_errorf(1, "shf_ungetc: flags %x", shf->flags);
+		internal_errorf("shf_ungetc: flags %x", shf->flags);
 
 	if ((shf->flags & SHF_ERROR) || c == EOF ||
 	    (shf->rp == shf->buf && shf->rnleft))
@@ -547,7 +547,7 @@ int
 shf_putchar(int c, struct shf *shf)
 {
 	if (!(shf->flags & SHF_WR))
-		internal_errorf(1, "shf_putchar: flags %x", shf->flags);
+		internal_errorf("shf_putchar: flags %x", shf->flags);
 
 	if (c == EOF)
 		return EOF;
@@ -557,7 +557,7 @@ shf_putchar(int c, struct shf *shf)
 		int n;
 
 		if (shf->fd < 0)
-			internal_errorf(1, "shf_putchar: no fd");
+			internal_errorf("shf_putchar: no fd");
 		if (shf->flags & SHF_ERROR) {
 			errno = shf->errno_;
 			return EOF;
@@ -603,10 +603,10 @@ shf_write(const char *buf, int nbytes, struct shf *shf)
 	int ncopy;
 
 	if (!(shf->flags & SHF_WR))
-		internal_errorf(1, "shf_write: flags %x", shf->flags);
+		internal_errorf("shf_write: flags %x", shf->flags);
 
 	if (nbytes < 0)
-		internal_errorf(1, "shf_write: nbytes %d", nbytes);
+		internal_errorf("shf_write: nbytes %d", nbytes);
 
 	/* Don't buffer if buffer is empty and we're writting a large amount. */
 	if ((ncopy = shf->wnleft) &&
@@ -676,8 +676,8 @@ shf_snprintf(char *buf, int bsize, const char *fmt, ...)
 	int n;
 
 	if (!buf || bsize <= 0)
-		internal_errorf(1, "shf_snprintf: buf %lx, bsize %d",
-			(long) buf, bsize);
+		internal_errorf("shf_snprintf: buf %lx, bsize %d",
+			(long)buf, bsize);
 
 	shf_sopen(buf, bsize, SHF_WR, &shf);
 	va_start(args, fmt);
@@ -707,26 +707,10 @@ shf_smprintf(const char *fmt, ...)
 #endif
 
 #define BUF_SIZE	128
-#define FPBUF_SIZE	(DMAXEXP+16)/* this must be >
-				 *	MAX(DMAXEXP, log10(pow(2, DSIGNIF)))
-				 *    + ceil(log10(DMAXEXP)) + 8 (I think).
-				 * Since this is hard to express as a
-				 * constant, just use a large buffer.
-				 */
-
-/*
- *	What kinda of machine we on?  Hopefully the C compiler will optimize
- *  this out...
- *
- *	For shorts, we want sign extend for %d but not for %[oxu] - on 16 bit
- *  machines it don't matter.  Assumes C compiler has converted shorts to
- *  ints before pushing them.
+/* must be > MAX(DMAXEXP, log10(pow(2, DSIGNIF))) + ceil(log10(DMAXEXP)) + 8
+ * (I think); since it's hard to express as a constant, just use a large buffer
  */
-#define POP_INT(f, s, a) \
-	(((f) & FL_LONG) ? va_arg((a), unsigned long) :			\
-	    (sizeof(int) < sizeof(long) ? ((s) ?			\
-	    (long) va_arg((a), int) : va_arg((a), unsigned)) :		\
-	    va_arg((a), unsigned)))
+#define FPBUF_SIZE	(DMAXEXP+16)
 
 #define	FL_HASH		0x001	/* '#' seen */
 #define FL_PLUS		0x002	/* '+' seen */
@@ -743,18 +727,14 @@ shf_smprintf(const char *fmt, ...)
 int
 shf_vfprintf(struct shf *shf, const char *fmt, va_list args)
 {
-	char		c, *s;
-	int		tmp = 0;
-	int		field, precision;
-	int		len;
-	int		flags;
-	unsigned long	lnum;
-					/* %#o produces the longest output */
-	char		numbuf[(8 * sizeof(long) + 2) / 3 + 1];
+	const char *s;
+	char c, *cp;
+	int tmp = 0, field, precision, len, flags;
+	unsigned long lnum;
+	/* %#o produces the longest output */
+	char numbuf[(8 * sizeof(long) + 2) / 3 + 1];
 	/* this stuff for dealing with the buffer */
-	int		nwritten = 0;
-
-	static char	nulls[] = "(null %s)";
+	int nwritten = 0;
 
 	if (!fmt)
 		return 0;
@@ -819,9 +799,9 @@ shf_vfprintf(struct shf *shf, const char *fmt, va_list args)
 				flags |= FL_SHORT;
 				continue;
 			}
-			if (digit(c)) {
+			if (ksh_isdigit(c)) {
 				tmp = c - '0';
-				while (c = *fmt++, digit(c))
+				while (c = *fmt++, ksh_isdigit(c))
 					tmp = tmp * 10 + c - '0';
 				--fmt;
 				if (tmp < 0)		/* overflow? */
@@ -849,8 +829,8 @@ shf_vfprintf(struct shf *shf, const char *fmt, va_list args)
 		switch (c) {
 		case 'p': /* pointer */
 			flags &= ~(FL_LONG | FL_SHORT);
-			if (sizeof(char *) > sizeof(int))
-				flags |= FL_LONG; /* hope it fits.. */
+			flags |= (sizeof(char *) > sizeof(int)) ?
+			    /* hope it fits.. */ FL_LONG : 0;
 			/* aaahhh... */
 		case 'd':
 		case 'i':
@@ -858,8 +838,22 @@ shf_vfprintf(struct shf *shf, const char *fmt, va_list args)
 		case 'u':
 		case 'x':
 			flags |= FL_NUMBER;
-			s = &numbuf[sizeof(numbuf)];
-			lnum = POP_INT(flags, c == 'd', args);
+			cp = &numbuf[sizeof (numbuf)];
+			/*-
+			 * XXX any better way to do this?
+			 * XXX hopefully the compiler optimises this out
+			 *
+			 * For shorts, we want sign extend for %d but not
+			 * for %[oxu] - on 16 bit machines it doesn't matter.
+			 * Assumes C compiler has converted shorts to ints
+			 * before pushing them.  XXX optimise this -tg
+			 */
+			if (flags & FL_LONG)
+				lnum = va_arg(args, unsigned long);
+			else if ((sizeof (int) < sizeof (long)) && (c == 'd'))
+				lnum = (long) va_arg(args, int);
+			else
+				lnum = va_arg(args, unsigned);
 			switch (c) {
 			case 'd':
 			case 'i':
@@ -871,28 +865,28 @@ shf_vfprintf(struct shf *shf, const char *fmt, va_list args)
 
 			case 'u':
 				do {
-					*--s = lnum % 10 + '0';
+					*--cp = lnum % 10 + '0';
 					lnum /= 10;
 				} while (lnum);
 
 				if (c != 'u') {
 					if (tmp)
-						*--s = '-';
+						*--cp = '-';
 					else if (flags & FL_PLUS)
-						*--s = '+';
+						*--cp = '+';
 					else if (flags & FL_BLANK)
-						*--s = ' ';
+						*--cp = ' ';
 				}
 				break;
 
 			case 'o':
 				do {
-					*--s = (lnum & 0x7) + '0';
+					*--cp = (lnum & 0x7) + '0';
 					lnum >>= 3;
 				} while (lnum);
 
-				if ((flags & FL_HASH) && *s != '0')
-					*--s = '0';
+				if ((flags & FL_HASH) && *cp != '0')
+					*--cp = '0';
 				break;
 
 			case 'p':
@@ -902,17 +896,17 @@ shf_vfprintf(struct shf *shf, const char *fmt, va_list args)
 				    "0123456789ABCDEF" :
 				    "0123456789abcdef";
 				do {
-					*--s = digits[lnum & 0xf];
+					*--cp = digits[lnum & 0xf];
 					lnum >>= 4;
 				} while (lnum);
 
 				if (flags & FL_HASH) {
-					*--s = (flags & FL_UPPER) ? 'X' : 'x';
-					*--s = '0';
+					*--cp = (flags & FL_UPPER) ? 'X' : 'x';
+					*--cp = '0';
 				}
 			    }
 			}
-			len = &numbuf[sizeof(numbuf)] - s;
+			len = &numbuf[sizeof (numbuf)] - (s = cp);
 			if (flags & FL_DOT) {
 				if (precision > len) {
 					field = precision;
@@ -923,8 +917,8 @@ shf_vfprintf(struct shf *shf, const char *fmt, va_list args)
 			break;
 
 		case 's':
-			if (!(s = va_arg(args, char *)))
-				s = nulls;
+			if (!(s = va_arg(args, const char *)))
+				s = "(null)";
 			len = strlen(s);
 			break;
 
@@ -944,9 +938,8 @@ shf_vfprintf(struct shf *shf, const char *fmt, va_list args)
 		}
 
 		/*
-		 *	At this point s should point to a string that is
-		 *  to be formatted, and len should be the length of the
-		 *  string.
+		 * At this point s should point to a string that is to be
+		 * formatted, and len should be the length of the string.
 		 */
 		if (!(flags & FL_DOT) || len < precision)
 			precision = len;
@@ -956,7 +949,8 @@ shf_vfprintf(struct shf *shf, const char *fmt, va_list args)
 				field = -field;
 				/* skip past sign or 0x when padding with 0 */
 				if ((flags & FL_ZERO) && (flags & FL_NUMBER)) {
-					if (*s == '+' || *s == '-' || *s ==' ') {
+					if (*s == '+' || *s == '-' ||
+					    *s == ' ') {
 						shf_putc(*s, shf);
 						s++;
 						precision--;
@@ -1000,3 +994,19 @@ shf_vfprintf(struct shf *shf, const char *fmt, va_list args)
 
 	return shf_error(shf) ? EOF : nwritten;
 }
+
+#ifdef MKSH_SMALL
+int
+shf_getc(struct shf *shf)
+{
+	return ((shf)->rnleft > 0 ? (shf)->rnleft--, *(shf)->rp++ :
+	    shf_getchar(shf));
+}
+
+int
+shf_putc(int c, struct shf *shf)
+{
+	return ((shf)->wnleft == 0 ? shf_putchar((c), (shf)) :
+	    ((shf)->wnleft--, *(shf)->wp++ = (c)));
+}
+#endif
