@@ -2,19 +2,19 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/syn.c,v 1.8 2006/08/01 13:43:28 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/syn.c,v 1.12.2.1 2007/05/13 19:29:40 tg Exp $");
 
 struct nesting_state {
-	int	start_token;	/* token than began nesting (eg, FOR) */
-	int	start_line;	/* line nesting began on */
+	int start_token;	/* token than began nesting (eg, FOR) */
+	int start_line;		/* line nesting began on */
 };
 
-static void	yyparse(void);
+static void yyparse(void);
 static struct op *pipeline(int);
 static struct op *andor(void);
 static struct op *c_list(int);
 static struct ioword *synio(int);
-static void	musthave(int, int);
+static void musthave(int, int);
 static struct op *nested(int, int, int);
 static struct op *get_command(int);
 static struct op *dogroup(void);
@@ -23,34 +23,32 @@ static struct op *elsepart(void);
 static struct op *caselist(void);
 static struct op *casepart(int);
 static struct op *function_body(char *, int);
-static char **	wordlist(void);
+static char **wordlist(void);
 static struct op *block(int, struct op *, struct op *, char **);
 static struct op *newtp(int);
-static void	syntaxerr(const char *) __attribute__((__noreturn__));
-static void	nesting_push(struct nesting_state *, int);
-static void	nesting_pop(struct nesting_state *);
-static int	assign_command(char *);
-static int	inalias(struct source *);
-static int	dbtestp_isa(Test_env *, Test_meta);
+static void syntaxerr(const char *)
+    __attribute__((noreturn));
+static void nesting_push(struct nesting_state *, int);
+static void nesting_pop(struct nesting_state *);
+static int assign_command(char *);
+static int inalias(struct source *);
+static int dbtestp_isa(Test_env *, Test_meta);
 static const char *dbtestp_getopnd(Test_env *, Test_op, int);
-static int	dbtestp_eval(Test_env *, Test_op, const char *,
+static int dbtestp_eval(Test_env *, Test_op, const char *,
     const char *, int);
 static void dbtestp_error(Test_env *, int, const char *)
     __attribute__((noreturn));
 
-static	struct	op	*outtree; /* yyparse output */
-
+static struct op *outtree;		/* yyparse output */
 static struct nesting_state nesting;	/* \n changed to ; */
 
-static	int	reject;		/* token(cf) gets symbol again */
-static	int	symbol;		/* yylex value */
+static int reject;		/* token(cf) gets symbol again */
+static int symbol;		/* yylex value */
 
-#define	REJECT	(reject = 1)
-#define	ACCEPT	(reject = 0)
-#define	token(cf) \
-	((reject) ? (ACCEPT, symbol) : (symbol = yylex(cf)))
-#define	tpeek(cf) \
-	((reject) ? (symbol) : (REJECT, symbol = yylex(cf)))
+#define	REJECT		(reject = 1)
+#define	ACCEPT		(reject = 0)
+#define	token(cf)	((reject) ? (ACCEPT, symbol) : (symbol = yylex(cf)))
+#define	tpeek(cf)	((reject) ? (symbol) : (REJECT, symbol = yylex(cf)))
 
 static void
 yyparse(void)
@@ -378,7 +376,7 @@ get_command(int cf)
 	}
 
 	if (iopn == 0) {
-		afree((void*) iops, ATEMP);
+		afree((void*)iops, ATEMP);
 		t->ioact = NULL;
 	} else {
 		iops[iopn++] = NULL;
@@ -389,7 +387,7 @@ get_command(int cf)
 
 	if (t->type == TCOM || t->type == TDBRACKET) {
 		XPput(args, NULL);
-		t->args = (char **) XPclose(args);
+		t->args = (const char **)XPclose(args);
 		XPput(vars, NULL);
 		t->vars = (char **) XPclose(vars);
 	} else {
@@ -551,6 +549,7 @@ function_body(char *name,
 	old_func_parse = e->flags & EF_FUNC_PARSE;
 	e->flags |= EF_FUNC_PARSE;
 	if ((t->left = get_command(CONTIN)) == NULL) {
+		char *tv;
 		/*
 		 * Probably something like foo() followed by eof or ;.
 		 * This is accepted by sh and ksh88.
@@ -558,13 +557,13 @@ function_body(char *name,
 		 * be used as input), we pretend there is a colon here.
 		 */
 		t->left = newtp(TCOM);
-		t->left->args = (char **) alloc(sizeof(char *) * 2, ATEMP);
-		t->left->args[0] = alloc(sizeof(char) * 3, ATEMP);
-		t->left->args[0][0] = CHAR;
-		t->left->args[0][1] = ':';
-		t->left->args[0][2] = EOS;
+		t->left->args = (const char **)alloc(sizeof(char *) * 2, ATEMP);
+		t->left->args[0] = tv = alloc(sizeof(char) * 3, ATEMP);
+		tv[0] = CHAR;
+		tv[1] = ':';
+		tv[2] = EOS;
 		t->left->args[1] = NULL;
-		t->left->vars = (char **) alloc(sizeof(char *), ATEMP);
+		t->left->vars = (char **)alloc(sizeof(char *), ATEMP);
 		t->left->vars[0] = NULL;
 		t->left->lineno = 1;
 	}
@@ -741,10 +740,11 @@ newtp(int type)
 {
 	struct op *t;
 
-	t = (struct op *) alloc(sizeof(*t), ATEMP);
+	t = (struct op *)alloc(sizeof(*t), ATEMP);
 	t->type = type;
 	t->u.evalflags = 0;
-	t->args = t->vars = NULL;
+	t->args = NULL;
+	t->vars = NULL;
 	t->ioact = NULL;
 	t->left = t->right = NULL;
 	t->str = NULL;
@@ -849,18 +849,16 @@ dbtestp_isa(Test_env *te, Test_meta meta)
 		ret = uqword && strcmp(yylval.cp, db_close) == 0;
 	if (ret) {
 		ACCEPT;
-		if (meta != TM_END) {
-			if (!save)
-				save = wdcopy(dbtest_tokens[(int) meta], ATEMP);
+		if (meta < NELEM(dbtest_tokens))
+			save = wdcopy(dbtest_tokens[(int)meta], ATEMP);
+		if (save)
 			XPput(*te->pos.av, save);
-		}
 	}
 	return ret;
 }
 
 static const char *
-dbtestp_getopnd(Test_env *te, Test_op op __attribute__((unused)),
-    int do_eval __attribute__((unused)))
+dbtestp_getopnd(Test_env *te, Test_op op __unused, int do_eval __unused)
 {
 	int c = tpeek(ARRAYVAR);
 
@@ -874,11 +872,9 @@ dbtestp_getopnd(Test_env *te, Test_op op __attribute__((unused)),
 }
 
 static int
-dbtestp_eval(Test_env *te __attribute__((unused)),
-    Test_op op __attribute__((unused)),
-    const char *opnd1 __attribute__((unused)),
-    const char *opnd2 __attribute__((unused)),
-    int do_eval __attribute__((unused)))
+dbtestp_eval(Test_env *te __unused, Test_op op __unused,
+    const char *opnd1 __unused, const char *opnd2 __unused,
+    int do_eval __unused)
 {
 	return 1;
 }
