@@ -2,27 +2,26 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/tree.c,v 1.4 2006/05/10 18:54:13 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/tree.c,v 1.9.2.1 2007/05/13 19:29:40 tg Exp $");
 
 #define INDENT	4
 
 #define tputc(c, shf)	shf_putchar(c, shf);
-static void	ptree(struct op *, int, struct shf *);
-static void	pioact(struct shf *, int, struct ioword *);
-static void	tputC(int, struct shf *);
-static void	tputS(char *, struct shf *);
-static void	vfptreef(struct shf *, int, const char *, va_list);
+static void ptree(struct op *, int, struct shf *);
+static void pioact(struct shf *, int, struct ioword *);
+static void tputC(int, struct shf *);
+static void tputS(char *, struct shf *);
+static void vfptreef(struct shf *, int, const char *, va_list);
 static struct ioword **iocopy(struct ioword **, Area *);
-static void     iofree(struct ioword **, Area *);
+static void iofree(struct ioword **, Area *);
 
 /*
  * print a command tree
  */
-
 static void
 ptree(struct op *t, int indent, struct shf *shf)
 {
-	char **w;
+	const char **w;
 	struct ioword **ioact;
 	struct op *t1;
 
@@ -32,7 +31,7 @@ ptree(struct op *t, int indent, struct shf *shf)
 	switch (t->type) {
 	case TCOM:
 		if (t->vars)
-			for (w = t->vars; *w != NULL; )
+			for (w = (const char **)t->vars; *w != NULL; )
 				fptreef(shf, indent, "%S ", *w++);
 		else
 			fptreef(shf, indent, "#no-vars# ");
@@ -83,7 +82,7 @@ ptree(struct op *t, int indent, struct shf *shf)
 			fptreef(shf, indent, "for %s ", t->str);
 		if (t->vars != NULL) {
 			fptreef(shf, indent, "in ");
-			for (w = t->vars; *w; )
+			for (w = (const char **)t->vars; *w; )
 				fptreef(shf, indent, "%S ", *w++);
 			fptreef(shf, indent, "%;");
 		}
@@ -94,7 +93,7 @@ ptree(struct op *t, int indent, struct shf *shf)
 		fptreef(shf, indent, "case %S in", t->str);
 		for (t1 = t->left; t1 != NULL; t1 = t1->right) {
 			fptreef(shf, indent, "%N(");
-			for (w = t1->vars; *w != NULL; w++)
+			for (w = (const char **)t1->vars; *w != NULL; w++)
 				fptreef(shf, indent, "%S%c", *w,
 				    (w[1] != NULL) ? '|' : ')');
 			fptreef(shf, indent + INDENT, "%;%T%N;;", t1->left);
@@ -240,7 +239,6 @@ pioact(struct shf *shf, int indent, struct ioword *iop)
 /*
  * variants of fputc, fputs for ptreef and snptreef
  */
-
 static void
 tputC(int c, struct shf *shf)
 {
@@ -369,32 +367,21 @@ vfptreef(struct shf *shf, int indent, const char *fmt, va_list va)
 
 	while ((c = *fmt++)) {
 		if (c == '%') {
-			long n;
-			char *p;
-			int neg;
-
 			switch ((c = *fmt++)) {
 			case 'c':
 				tputc(va_arg(va, int), shf);
 				break;
 			case 's':
-				p = va_arg(va, char *);
-				while (*p)
-					tputc(*p++, shf);
+				shf_puts(va_arg(va, char *), shf);
 				break;
 			case 'S':	/* word */
-				p = va_arg(va, char *);
-				tputS(p, shf);
+				tputS(va_arg(va, char *), shf);
 				break;
-			case 'd': case 'u': /* decimal */
-				n = (c == 'd') ? (long)va_arg(va, int) :
-				    (long)va_arg(va, unsigned int);
-				neg = c=='d' && n<0;
-				p = ulton((neg) ? -n : n, 10);
-				if (neg)
-					*--p = '-';
-				while (*p)
-					tputc(*p++, shf);
+			case 'd':	/* decimal */
+				shf_fprintf(shf, "%d", va_arg(va, int));
+				break;
+			case 'u':	/* decimal */
+				shf_fprintf(shf, "%u", va_arg(va, unsigned));
 				break;
 			case 'T':	/* format tree */
 				ptree(va_arg(va, struct op *), indent, shf);
@@ -430,12 +417,12 @@ vfptreef(struct shf *shf, int indent, const char *fmt, va_list va)
 /*
  * copy tree (for function definition)
  */
-
 struct op *
 tcopy(struct op *t, Area *ap)
 {
 	struct op *r;
-	char **tw, **rw;
+	const char **tw;
+	char **rw;
 
 	if (t == NULL)
 		return NULL;
@@ -450,11 +437,11 @@ tcopy(struct op *t, Area *ap)
 	if (t->vars == NULL)
 		r->vars = NULL;
 	else {
-		for (tw = t->vars; *tw++ != NULL; )
+		for (tw = (const char **)t->vars; *tw++ != NULL; )
 			;
 		rw = r->vars = (char **)
-			alloc((tw - t->vars + 1) * sizeof(*tw), ap);
-		for (tw = t->vars; *tw != NULL; )
+		    alloc((tw - (const char **)t->vars + 1) * sizeof(*tw), ap);
+		for (tw = (const char **)t->vars; *tw != NULL; )
 			*rw++ = wdcopy(*tw++, ap);
 		*rw = NULL;
 	}
@@ -464,8 +451,8 @@ tcopy(struct op *t, Area *ap)
 	else {
 		for (tw = t->args; *tw++ != NULL; )
 			;
-		rw = r->args = (char **)
-			alloc((tw - t->args + 1) * sizeof(*tw), ap);
+		r->args = (const char **)(rw = (char **)
+			alloc((tw - t->args + 1) * sizeof(*tw), ap));
 		for (tw = t->args; *tw != NULL; )
 			*rw++ = wdcopy(*tw++, ap);
 		*rw = NULL;
@@ -488,7 +475,7 @@ wdcopy(const char *wp, Area *ap)
 }
 
 /* return the position of prefix c in wp plus 1 */
-char *
+const char *
 wdscan(const char *wp, int c)
 {
 	int nest = 0;
@@ -496,7 +483,7 @@ wdscan(const char *wp, int c)
 	while (1)
 		switch (*wp++) {
 		case EOS:
-			return (char *) wp;
+			return (wp);
 		case CHAR:
 		case QCHAR:
 			wp++;
@@ -517,7 +504,7 @@ wdscan(const char *wp, int c)
 		case CSUBST:
 			wp++;
 			if (c == CSUBST && nest == 0)
-				return (char *) wp;
+				return (wp);
 			nest--;
 			break;
 		case OPAT:
@@ -527,12 +514,12 @@ wdscan(const char *wp, int c)
 		case SPAT:
 		case CPAT:
 			if (c == wp[-1] && nest == 0)
-				return (char *) wp;
+				return (wp);
 			if (wp[-1] == CPAT)
 				nest--;
 			break;
 		default:
-			internal_errorf(0,
+			internal_warningf(
 			    "wdscan: unknown char 0x%x (carrying on)",
 			    wp[-1]);
 		}
@@ -607,7 +594,7 @@ wdstrip(const char *wp)
 		}
 }
 
-static	struct ioword **
+static struct ioword **
 iocopy(struct ioword **iow, Area *ap)
 {
 	struct ioword **ior;
@@ -639,7 +626,6 @@ iocopy(struct ioword **iow, Area *ap)
 /*
  * free tree (for function definition)
  */
-
 void
 tfree(struct op *t, Area *ap)
 {
@@ -658,7 +644,10 @@ tfree(struct op *t, Area *ap)
 	}
 
 	if (t->args != NULL) {
-		for (w = t->args; *w != NULL; w++)
+		union mksh_ccphack cw;
+		/* XXX we assume the caller is right */
+		cw.ro = t->args;
+		for (w = cw.rw; *w != NULL; w++)
 			afree((void*)*w, ap);
 		afree((void*)t->args, ap);
 	}
@@ -672,7 +661,7 @@ tfree(struct op *t, Area *ap)
 	afree((void*)t, ap);
 }
 
-static	void
+static void
 iofree(struct ioword **iow, Area *ap)
 {
 	struct ioword **iop;
