@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/var.c,v 1.23 2006/08/18 18:48:26 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/var.c,v 1.37.2.1 2007/05/13 19:29:40 tg Exp $");
 
 /*
  * Variables
@@ -32,14 +32,14 @@ void
 newblock(void)
 {
 	struct block *l;
-	static char *const empty[] = {null};
+	static const char *empty[] = { null };
 
 	l = (struct block *) alloc(sizeof(struct block), ATEMP);
 	l->flags = 0;
 	ainit(&l->area); /* todo: could use e->area (l->area => l->areap) */
 	if (!e->loc) {
 		l->argc = 0;
-		l->argv = (char **) empty;
+		l->argv = empty;
 	} else {
 		l->argc = e->loc->argc;
 		l->argv = e->loc->argv;
@@ -75,7 +75,7 @@ popblock(void)
 	afree(l, ATEMP);
 }
 
-/* called by main() to initialize variable data structures */
+/* called by main() to initialise variable data structures */
 void
 initvar(void)
 {
@@ -88,7 +88,9 @@ initvar(void)
 		{ "OPTIND",		V_OPTIND },
 		{ "PATH",		V_PATH },
 		{ "TMPDIR",		V_TMPDIR },
+#if HAVE_PERSISTENT_HISTORY
 		{ "HISTFILE",		V_HISTFILE },
+#endif
 		{ "HISTSIZE",		V_HISTSIZE },
 		{ "RANDOM",		V_RANDOM },
 		{ "SECONDS",		V_SECONDS },
@@ -155,7 +157,7 @@ global(const char *n)
 	n = array_index_calc(n, &array, &val);
 	h = hash(n);
 	c = n[0];
-	if (!letter(c)) {
+	if (!ksh_isalphx(c)) {
 		if (array)
 			errorf("bad substitution");
 		vp = &vtemp;
@@ -163,8 +165,8 @@ global(const char *n)
 		vp->type = 0;
 		vp->areap = ATEMP;
 		*vp->name = c;
-		if (digit(c)) {
-			for (c = 0; digit(*n); n++)
+		if (ksh_isdigit(c)) {
+			for (c = 0; ksh_isdigit(*n); n++)
 				c = c*10 + *n-'0';
 			if (c <= l->argc)
 				/* setstr can't fail here */
@@ -235,7 +237,7 @@ local(const char *n, bool copy)
 	/* Check to see if this is an array */
 	n = array_index_calc(n, &array, &val);
 	h = hash(n);
-	if (!letter(*n)) {
+	if (!ksh_isalphx(*n)) {
 		vp = &vtemp;
 		vp->flag = DEFINED|RDONLY;
 		vp->type = 0;
@@ -347,7 +349,7 @@ setstr(struct tbl *vq, const char *s, int error_ok)
 			/* debugging */
 			if (s >= vq->val.s &&
 			    s <= vq->val.s + strlen(vq->val.s))
-				internal_errorf(true,
+				internal_errorf(
 				    "setstr: %s=%s: assigning to self",
 				    vq->name, s);
 			afree((void*)vq->val.s, vq->areap);
@@ -430,26 +432,25 @@ getint(struct tbl *vp, long int *nump, bool arith)
 	for (c = *s++; c ; c = *s++) {
 		if (c == '-') {
 			neg++;
+			continue;
 		} else if (c == '#') {
 			base = (int) num;
 			if (have_base || base < 2 || base > 36)
 				return -1;
 			num = 0;
 			have_base = 1;
-		} else if (letnum(c)) {
-			if (isdigit(c))
-				c -= '0';
-			else if (islower(c))
-				c -= 'a' - 10; /* todo: assumes ascii */
-			else if (isupper(c))
-				c -= 'A' - 10; /* todo: assumes ascii */
-			else
-				c = -1; /* _: force error */
-			if (c < 0 || c >= base)
-				return -1;
-			num = num * base + c;
-		} else
+			continue;
+		} else if (ksh_isdigit(c))
+			c -= '0';
+		else if (ksh_islower(c))
+			c -= 'a' - 10;
+		else if (ksh_isupper(c))
+			c -= 'A' - 10;
+		else
 			return -1;
+		if (c < 0 || c >= base)
+			return -1;
+		num = num * base + c;
 	}
 	if (neg)
 		num = -num;
@@ -503,20 +504,22 @@ formatstr(struct tbl *vp, const char *s)
 		if (vp->flag & RJUST) {
 			const char *qq = s + olen;
 			/* strip trailing spaces (at&t uses qq[-1] == ' ') */
-			while (qq > s && isspace((unsigned char)qq[-1]))
+			while (qq > s && ksh_isspace(qq[-1]))
 				--qq;
 			slen = qq - s;
 			if (slen > vp->u2.field) {
 				s += slen - vp->u2.field;
 				slen = vp->u2.field;
 			}
-			shf_snprintf(p, nlen + 1,
-				((vp->flag & ZEROFIL) && digit(*s)) ?
-					  "%0*s%.*s" : "%*s%.*s",
-				vp->u2.field - slen, null, slen, s);
+			if (vp->u2.field - slen)
+				memset(p, (vp->flag & ZEROFIL) ? '0' : ' ',
+				    vp->u2.field - slen);
+			shf_snprintf(p + vp->u2.field - slen,
+			    nlen + 1 - (vp->u2.field - slen),
+			    "%.*s", slen, s);
 		} else {
 			/* strip leading spaces/zeros */
-			while (isspace((unsigned char)*s))
+			while (ksh_isspace(*s))
 				s++;
 			if (vp->flag & ZEROFIL)
 				while (*s == '0')
@@ -529,12 +532,10 @@ formatstr(struct tbl *vp, const char *s)
 
 	if (vp->flag & UCASEV_AL) {
 		for (q = p; *q; q++)
-			if (islower((unsigned char)*q))
-				*q = toupper((unsigned char)*q);
+			*q = ksh_toupper(*q);
 	} else if (vp->flag & LCASEV) {
 		for (q = p; *q; q++)
-			if (isupper((unsigned char)*q))
-				*q = tolower((unsigned char)*q);
+			*q = ksh_tolower(*q);
 	}
 
 	return p;
@@ -594,7 +595,7 @@ typeset(const char *var, Tflag set, Tflag clr, int field, int base)
 		if (set & IMPORT) {
 			int i;
 			for (i = 1; i < len - 1; i++)
-				if (!digit(val[i]))
+				if (!ksh_isdigit(val[i]))
 					return NULL;
 		}
 		val += len;
@@ -605,7 +606,7 @@ typeset(const char *var, Tflag set, Tflag clr, int field, int base)
 		/* Importing from original environment: must have an = */
 		if (set & IMPORT)
 			return NULL;
-		tvar = (char *) var;
+		tvar = str_save(var, ATEMP);
 		val = NULL;
 	}
 
@@ -627,8 +628,7 @@ typeset(const char *var, Tflag set, Tflag clr, int field, int base)
 	    (val || clr || (set & ~EXPORT)))
 		/* XXX check calls - is error here ok by POSIX? */
 		errorf("%s: is read only", tvar);
-	if (val)
-		afree(tvar, ATEMP);
+	afree(tvar, ATEMP);
 
 	/* most calls are with set/clr == 0 */
 	if (set | clr) {
@@ -744,29 +744,29 @@ unset(struct tbl *vp, int array_ref)
  * argument if there is no legal name, returns * a pointer to the terminating
  * null if whole string is legal).
  */
-char *
+const char *
 skip_varname(const char *s, int aok)
 {
 	int alen;
 
-	if (s && letter(*s)) {
-		while (*++s && letnum(*s))
+	if (s && ksh_isalphx(*s)) {
+		while (*++s && ksh_isalnux(*s))
 			;
 		if (aok && *s == '[' && (alen = array_ref_len(s)))
 			s += alen;
 	}
-	return (char *) s;
+	return (s);
 }
 
 /* Return a pointer to the first character past any legal variable name.  */
-char *
+const char *
 skip_wdvarname(const char *s,
     int aok)				/* skip array de-reference? */
 {
-	if (s[0] == CHAR && letter(s[1])) {
+	if (s[0] == CHAR && ksh_isalphx(s[1])) {
 		do {
 			s += 2;
-		} while (s[0] == CHAR && letnum(s[1]));
+		} while (s[0] == CHAR && ksh_isalnux(s[1]));
 		if (aok && s[0] == CHAR && s[1] == '[') {
 			/* skip possible array de-reference */
 			const char *p = s;
@@ -787,14 +787,14 @@ skip_wdvarname(const char *s,
 			}
 		}
 	}
-	return (char *) s;
+	return (s);
 }
 
 /* Check if coded string s is a variable name */
 int
 is_wdvarname(const char *s, int aok)
 {
-	char *p = skip_wdvarname(s, aok);
+	const char *p = skip_wdvarname(s, aok);
 
 	return p != s && p[0] == EOS;
 }
@@ -803,7 +803,7 @@ is_wdvarname(const char *s, int aok)
 int
 is_wdvarassign(const char *s)
 {
-	char *p = skip_wdvarname(s, true);
+	const char *p = skip_wdvarname(s, true);
 
 	return p != s && p[0] == CHAR && p[1] == '=';
 }
@@ -853,7 +853,7 @@ makenv(void)
  * we return values from rand() instead of arc4random()
  */
 #if HAVE_ARC4RANDOM
-int use_rand = 0;
+static int use_rand = 0;
 #endif
 
 /*
@@ -908,20 +908,22 @@ getspec(struct tbl *vp)
 		 * has been set - don't do anything in this case
 		 * (see initcoms[] in main.c).
 		 */
-		if (vp->flag & ISSET)
-			setint(vp, (long) (time(NULL) - seconds));
+		if (vp->flag & ISSET) {
+			struct timeval tv;
+
+			gettimeofday(&tv, NULL);
+			setint(vp, tv.tv_sec - seconds);
+		}
 		vp->flag |= SPECIAL;
 		break;
 	case V_RANDOM:
 		vp->flag &= ~SPECIAL;
 #if HAVE_ARC4RANDOM
-		if (use_rand)
-#endif
-			setint(vp, (long) (rand() & 0x7FFF));
-#if HAVE_ARC4RANDOM
+		if (!use_rand)
+			setint(vp, arc4random() & 0x7FFF);
 		else
-			setint(vp, arc4random() & 0x7FFFFFFF);
 #endif
+			setint(vp, rand() & 0x7FFF);
 		vp->flag |= SPECIAL;
 		break;
 	case V_HISTSIZE:
@@ -975,7 +977,7 @@ setspec(struct tbl *vp)
 			struct stat statb;
 
 			s = str_val(vp);
-			if (s[0] == '/' && eaccess(s, W_OK|X_OK) == 0 &&
+			if (s[0] == '/' && access(s, W_OK|X_OK) == 0 &&
 			    stat(s, &statb) == 0 && S_ISDIR(statb.st_mode))
 				tmpdir = str_save(s, APERM);
 		}
@@ -985,9 +987,11 @@ setspec(struct tbl *vp)
 		sethistsize((int) intval(vp));
 		vp->flag |= SPECIAL;
 		break;
+#if HAVE_PERSISTENT_HISTORY
 	case V_HISTFILE:
 		sethistfile(str_val(vp));
 		break;
+#endif
 	case V_COLUMNS:
 		if ((x_cols = intval(vp)) <= MIN_COLS)
 			x_cols = MIN_COLS;
@@ -1005,7 +1009,12 @@ setspec(struct tbl *vp)
 		break;
 	case V_SECONDS:
 		vp->flag &= ~SPECIAL;
-		seconds = time(NULL) - intval(vp);
+		{
+			struct timeval tv;
+
+			gettimeofday(&tv, NULL);
+			seconds = tv.tv_sec - intval(vp);
+		}
 		vp->flag |= SPECIAL;
 		break;
 	case V_TMOUT:
@@ -1130,9 +1139,9 @@ arrayname(const char *str)
 {
 	const char *p;
 
-	if ((p = strchr(str, '[')) == 0)
+	if ((p = cstrchr(str, '[')) == 0)
 		/* Shouldn't happen, but why worry? */
-		return (char *) str;
+		return str_save(str, ATEMP);
 
 	return str_nsave(str, p - str, ATEMP);
 }
@@ -1140,7 +1149,7 @@ arrayname(const char *str)
 /* Set (or overwrite, if !reset) the array variable var to the values in vals.
  */
 void
-set_array(const char *var, int reset, char **vals)
+set_array(const char *var, int reset, const char **vals)
 {
 	struct tbl *vp, *vq;
 	int i;
@@ -1165,26 +1174,3 @@ set_array(const char *var, int reset, char **vals)
 		setstr(vq, vals[i], KSH_RETURN_ERROR);
 	}
 }
-
-int
-eaccess(const char *pathname, int mode)
-{
-	bool need_uid, need_gid;
-	int rv, _errno;
-
-	if ((need_gid = (kshgid != kshegid)))
-		setregid(kshegid, kshgid);
-	if ((need_uid = (kshuid != ksheuid)))
-		setreuid(ksheuid, kshuid);
-
-	rv = access(pathname, mode);
-	_errno = errno;
-
-	if (need_gid)
-		setregid(kshgid, kshegid);
-	if (need_uid)
-		setreuid(kshuid, ksheuid);
-
-	errno = _errno;
-	return (rv);
-}
